# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.network import base
from tempest.common import identity
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators


class QuotasTest(base.BaseAdminNetworkTest):
    """Tests the following operations in the Neutron API:

        list quotas for projects who have non-default quota values
        show quotas for a specified project
        update quotas for a specified project
        reset quotas to default values for a specified project

    v2.0 of the API is assumed.
    It is also assumed that the per-project quota extension API is configured
    in /etc/neutron/neutron.conf as follows:

        quota_driver = neutron.db.quota.driver.DbQuotaDriver
    """

    @classmethod
    def skip_checks(cls):
        super(QuotasTest, cls).skip_checks()
        if not utils.is_extension_enabled('quotas', 'network'):
            msg = "quotas extension not enabled."
            raise cls.skipException(msg)

    def _check_quotas(self, new_quotas):
        # Add a project to conduct the test
        project = data_utils.rand_name('test_project_')
        description = data_utils.rand_name('desc_')
        project = identity.identity_utils(self.os_admin).create_project(
            name=project, description=description)
        project_id = project['id']
        self.addCleanup(identity.identity_utils(self.os_admin).delete_project,
                        project_id)

        # Change quotas for project
        quota_set = self.admin_quotas_client.update_quotas(
            project_id, **new_quotas)['quota']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_quotas_client.reset_quotas, project_id)
        for key, value in new_quotas.items():
            self.assertEqual(value, quota_set[key])

        # Confirm our project is listed among projects with non default quotas
        non_default_quotas = self.admin_quotas_client.list_quotas()
        found = False
        for qs in non_default_quotas['quotas']:
            if qs['project_id'] == project_id:
                found = True
        self.assertTrue(found)

        # Confirm from API quotas were changed as requested for project
        quota_set = self.admin_quotas_client.show_quotas(project_id)
        quota_set = quota_set['quota']
        for key, value in new_quotas.items():
            self.assertEqual(value, quota_set[key])

        # Reset quotas to default and confirm
        self.admin_quotas_client.reset_quotas(project_id)
        non_default_quotas = self.admin_quotas_client.list_quotas()
        for q in non_default_quotas['quotas']:
            self.assertNotEqual(project_id, q['project_id'])
        quota_set = self.admin_quotas_client.show_quotas(project_id)['quota']
        default_quotas = self.admin_quotas_client.show_default_quotas(
            project_id)['quota']
        self.assertEqual(default_quotas, quota_set)

    @decorators.idempotent_id('2390f766-836d-40ef-9aeb-e810d78207fb')
    def test_quotas(self):
        """Test update/list/show/reset of network quotas"""
        new_quotas = {'network': 0, 'port': 0}
        self._check_quotas(new_quotas)

    @testtools.skipUnless(utils.is_extension_enabled(
        'quota_details', 'network'), 'Quota details extension not enabled.')
    @decorators.idempotent_id('7b05ec5f-bf44-43cb-b28f-ddd72a824288')
    def test_show_quota_details(self):
        """Test showing network quota details"""
        # Show quota details for an existing project
        quota_details = self.admin_quotas_client.show_quota_details(
            self.admin_quotas_client.tenant_id)['quota']
        expected_keys = ['used', 'limit', 'reserved']
        for resource_type in quota_details:
            for key in expected_keys:
                self.assertIn(key, quota_details[resource_type])



# Copyright 2018 AT&T Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import agents_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestAgentsClient(base.BaseServiceTest):

    FAKE_AGENT_ID = "d32019d3-bc6e-4319-9c1d-6123f4135a88"

    FAKE_LIST_DATA = {
        "agents": [
            {
                "binary": "neutron-dhcp-agent",
                "description": None,
                "availability_zone": "nova",
                "heartbeat_timestamp": "2017-09-12 19:39:56",
                "admin_state_up": True,
                "alive": True,
                "id": "840d5d68-5759-4e9e-812f",
                "topic": "dhcp_agent",
                "host": "agenthost1",
                "agent_type": "DHCP agent",
                "started_at": "2017-09-12 19:35:36",
                "created_at": "2017-09-12 19:35:36",
                "resources_synced": None,
                "configurations": {
                    "subnets": 2,
                    "dhcp_lease_duration": 86400,
                    "dhcp_driver": "neutron.agent",
                    "networks": 1,
                    "log_agent_heartbeats": False,
                    "ports": 3
                }
            }
        ]
    }

    FAKE_SHOW_DATA = {
        "agent": {
            "binary": "neutron-openvswitch-agent",
            "description": None,
            "availability_zone": None,
            "heartbeat_timestamp": "2017-09-12 19:40:38",
            "admin_state_up": True,
            "alive": True,
            "id": "04c62b91-b799-48b7-9cd5-2982db6df9c6",
            "topic": "N/A",
            "host": "agenthost1",
            "agent_type": "Open vSwitch agent",
            "started_at": "2017-09-12 19:35:38",
            "created_at": "2017-09-12 19:35:38",
            "resources_synced": True,
            "configurations": {
                "ovs_hybrid_plug": True,
                "in_distributed_mode": False,
                "datapath_type": "system",
                "vhostuser_socket_dir": "/var/run/openvswitch",
                "tunneling_ip": "172.16.78.191",
                "arp_responder_enabled": False,
                "devices": 0,
                "ovs_capabilities": {
                    "datapath_types": [
                        "netdev",
                        "system"
                    ],
                    "iface_types": [
                        "geneve",
                        "gre",
                        "internal",
                        "ipsec_gre",
                        "lisp",
                        "patch",
                        "stt",
                        "system",
                        "tap",
                        "vxlan"
                    ]
                },
                "log_agent_heartbeats": False,
                "l2_population": False,
                "tunnel_types": [
                    "vxlan"
                ],
                "extensions": [],
                "enable_distributed_routing": False,
                "bridge_mappings": {
                    "public": "br-ex"
                }
            }
        }
    }

    FAKE_UPDATE_DATA = {
        "agent": {
            "description": "My OVS agent for OpenStack"
        }
    }

    def setUp(self):
        super(TestAgentsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.agents_client = agents_client.AgentsClient(
            fake_auth, "network", "regionOne")

    def _test_show_agent(self, bytes_body=False):
        self.check_service_client_function(
            self.agents_client.show_agent,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_SHOW_DATA,
            bytes_body,
            status=200,
            agent_id=self.FAKE_AGENT_ID)

    def _test_update_agent(self, bytes_body=False):
        self.check_service_client_function(
            self.agents_client.update_agent,
            "tempest.lib.common.rest_client.RestClient.put",
            self.FAKE_UPDATE_DATA,
            bytes_body,
            status=200,
            agent_id=self.FAKE_AGENT_ID)

    def _test_list_agents(self, bytes_body=False):
        self.check_service_client_function(
            self.agents_client.list_agents,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_LIST_DATA,
            bytes_body,
            status=200)

    def test_show_agent_with_str_body(self):
        self._test_show_agent()

    def test_show_agent_with_bytes_body(self):
        self._test_show_agent(bytes_body=True)

    def test_update_agent_with_str_body(self):
        self._test_update_agent()

    def test_update_agent_with_bytes_body(self):
        self._test_update_agent(bytes_body=True)

    def test_list_agent_with_str_body(self):
        self._test_list_agents()

    def test_list_agent_with_bytes_body(self):
        self._test_list_agents(bytes_body=True)

    def test_delete_agent(self):
        self.check_service_client_function(
            self.agents_client.delete_agent,
            "tempest.lib.common.rest_client.RestClient.delete",
            {},
            status=204,
            agent_id=self.FAKE_AGENT_ID)



# Copyright 2012 OpenStack Foundation
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import server_groups \
    as schema
from tempest.lib.api_schema.response.compute.v2_13 import server_groups \
    as schemav213
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class ServerGroupsClient(base_compute_client.BaseComputeClient):

    schema_versions_info = [
        {'min': None, 'max': '2.12', 'schema': schema},
        {'min': '2.13', 'max': None, 'schema': schemav213}]

    def create_server_group(self, **kwargs):
        """Create the server group.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-server-group
        """
        post_body = json.dumps({'server_group': kwargs})
        resp, body = self.post('os-server-groups', post_body)

        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.create_show_server_group, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_server_group(self, server_group_id):
        """Delete the given server-group."""
        resp, body = self.delete("os-server-groups/%s" % server_group_id)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.delete_server_group, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_server_groups(self):
        """List the server-groups."""
        resp, body = self.get("os-server-groups")
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.list_server_groups, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_server_group(self, server_group_id):
        """Get the details of given server_group."""
        resp, body = self.get("os-server-groups/%s" % server_group_id)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.create_show_server_group, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


# TODO(mriedem): Remove this test class once the nova queens branch goes into
# extended maintenance mode.
class VirtualInterfacesNegativeTestJSON(base.BaseV2ComputeTest):
    """Negative tests of virtual interfaces API

    Negative tests of virtual interfaces API for compute microversion less
    than 2.44.
    """

    max_microversion = '2.43'

    depends_on_nova_network = True

    @classmethod
    def setup_credentials(cls):
        # For this test no network resources are needed
        cls.set_network_resources()
        super(VirtualInterfacesNegativeTestJSON, cls).setup_credentials()

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('64ebd03c-1089-4306-93fa-60f5eb5c803c')
    @utils.services('network')
    def test_list_virtual_interfaces_invalid_server_id(self):
        """Test listing virtual interfaces of an invalid server should fail"""
        invalid_server_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.servers_client.list_virtual_interfaces,
                          invalid_server_id)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

common_security_group_rule = {
    'from_port': {'type': ['integer', 'null']},
    'to_port': {'type': ['integer', 'null']},
    'group': {
        'type': 'object',
        'properties': {
            'tenant_id': {'type': 'string'},
            'name': {'type': 'string'}
        },
        'additionalProperties': False,
    },
    'ip_protocol': {'type': ['string', 'null']},
    # 'parent_group_id' can be UUID so defining it as 'string' also.
    'parent_group_id': {'type': ['string', 'integer', 'null']},
    'ip_range': {
        'type': 'object',
        'properties': {
            'cidr': {'type': 'string'}
        },
        'additionalProperties': False,
        # When optional argument is provided in request body
        # like 'group_id' then, attribute 'cidr' does not
        # comes in response body. So it is not 'required'.
    },
    'id': {'type': ['string', 'integer']}
}

common_security_group = {
    'type': 'object',
    'properties': {
        'id': {'type': ['integer', 'string']},
        'name': {'type': 'string'},
        'tenant_id': {'type': 'string'},
        'rules': {
            'type': 'array',
            'items': {
                'type': ['object', 'null'],
                'properties': common_security_group_rule,
                'additionalProperties': False,
            }
        },
        'description': {'type': 'string'},
    },
    'additionalProperties': False,
    'required': ['id', 'name', 'tenant_id', 'rules', 'description'],
}

list_security_groups = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'security_groups': {
                'type': 'array',
                'items': common_security_group
            }
        },
        'additionalProperties': False,
        'required': ['security_groups']
    }
}

get_security_group = create_security_group = update_security_group = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'security_group': common_security_group
        },
        'additionalProperties': False,
        'required': ['security_group']
    }
}

delete_security_group = {
    'status_code': [202]
}

create_security_group_rule = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'security_group_rule': {
                'type': 'object',
                'properties': common_security_group_rule,
                'additionalProperties': False,
                'required': ['from_port', 'to_port', 'group', 'ip_protocol',
                             'parent_group_id', 'id', 'ip_range']
            }
        },
        'additionalProperties': False,
        'required': ['security_group_rule']
    }
}

delete_security_group_rule = {
    'status_code': [202]
}



# Copyright 2016 Rackspace
#
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
subunit-describe-calls is a parser for subunit streams to determine what REST
API calls are made inside of a test and in what order they are called.

Runtime Arguments
-----------------

* ``--subunit, -s``: (Optional) The path to the subunit file being parsed,
  defaults to stdin
* ``--non-subunit-name, -n``: (Optional) The file_name that the logs are being
  stored in
* ``--output-file, -o``: (Optional) The path where the JSON output will be
  written to. This contains more information than is present in stdout.
* ``--ports, -p``: (Optional) The path to a JSON file describing the ports
  being used by different services
* ``--verbose, -v``: (Optional) Print Request and Response Headers and Body
  data to stdout in the non cliff deprecated CLI
* ``--all-stdout, -a``: (Optional) Print Request and Response Headers and Body
  data to stdout


Usage
-----

subunit-describe-calls will take in either stdin subunit v1 or v2 stream or a
file path which contains either a subunit v1 or v2 stream passed via the
``--subunit`` parameter. This is then parsed checking for details contained in
the file_bytes of the ``--non-subunit-name`` parameter (the default is
pythonlogging which is what Tempest uses to store logs). By default `the
OpenStack default ports
<https://docs.openstack.org/install-guide/firewalls-default-ports.html>`_
are used unless a file is provided via the ``--ports`` option. The resulting
output is dumped in JSON output to the path provided in the ``--output-file``
option.

Ports file JSON structure
^^^^^^^^^^^^^^^^^^^^^^^^^
::

  {
      "<port number>": "<name of service>",
      ...
  }


Output file JSON structure
^^^^^^^^^^^^^^^^^^^^^^^^^^
::

  {
      "full_test_name[with_id_and_tags]": [
          {
              "name": "The ClassName.MethodName that made the call",
              "verb": "HTTP Verb",
              "service": "Name of the service",
              "url": "A shortened version of the URL called",
              "status_code": "The status code of the response",
              "request_headers": "The headers of the request",
              "request_body": "The body of the request",
              "response_headers": "The headers of the response",
              "response_body": "The body of the response"
          }
      ]
  }
"""
import argparse
import collections
import io
import os
import re
import sys
import traceback

from cliff.command import Command
from oslo_serialization import jsonutils as json
import subunit
import testtools


DESCRIPTION = "Outputs all HTTP calls a given test made that were logged."


class UrlParser(testtools.TestResult):

    uuid_re = re.compile(r'(^|[^0-9a-f])[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-'
                         '[0-9a-f]{4}-[0-9a-f]{12}([^0-9a-f]|$)')
    id_re = re.compile(r'(^|[^0-9a-z])[0-9a-z]{8}[0-9a-z]{4}[0-9a-z]{4}'
                       '[0-9a-z]{4}[0-9a-z]{12}([^0-9a-z]|$)')
    ip_re = re.compile(r'(^|[^0-9])[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]'
                       '{1,3}([^0-9]|$)')
    url_re = re.compile(r'.*INFO.*Request \((?P<name>.*)\): (?P<code>[\d]{3}) '
                        r'(?P<verb>\w*) (?P<url>.*) .*')
    port_re = re.compile(r'.*:(?P<port>\d+).*')
    path_re = re.compile(r'http[s]?://[^/]*/(?P<path>.*)')
    request_re = re.compile(r'.* Request - Headers: (?P<headers>.*)')
    response_re = re.compile(r'.* Response - Headers: (?P<headers>.*)')
    body_re = re.compile(r'.*Body: (?P<body>.*)')

    # Based on OpenStack default ports:
    # https://docs.openstack.org/install-guide/firewalls-default-ports.html
    services = {
        "8776": "Block Storage",
        "8774": "Nova",
        "8773": "Nova-API", "8775": "Nova-API",
        "8386": "Sahara",
        "35357": "Keystone", "5000": "Keystone",
        "9292": "Glance", "9191": "Glance",
        "9696": "Neutron",
        "6000": "Swift", "6001": "Swift", "6002": "Swift",
        "8004": "Heat", "8000": "Heat", "8003": "Heat",
        "8777": "Ceilometer",
        "80": "Horizon",
        "8080": "Swift",
        "443": "SSL",
        "873": "rsync",
        "3260": "iSCSI",
        "3306": "MySQL",
        "5672": "AMQP",
        "8082": "murano",
        "8778": "Clustering",
        "8999": "Vitrage",
        "8989": "Mistral"}

    def __init__(self, services=None):
        super(UrlParser, self).__init__()
        self.test_logs = {}
        self.services = services or self.services

    def addSuccess(self, test, details=None):
        output = test.shortDescription() or test.id()
        calls = self.parse_details(details)
        self.test_logs.update({output: calls})

    def addSkip(self, test, err, details=None):
        output = test.shortDescription() or test.id()
        calls = self.parse_details(details)
        self.test_logs.update({output: calls})

    def addError(self, test, err, details=None):
        output = test.shortDescription() or test.id()
        calls = self.parse_details(details)
        self.test_logs.update({output: calls})

    def addFailure(self, test, err, details=None):
        output = test.shortDescription() or test.id()
        calls = self.parse_details(details)
        self.test_logs.update({output: calls})

    def stopTestRun(self):
        super(UrlParser, self).stopTestRun()

    def startTestRun(self):
        super(UrlParser, self).startTestRun()

    def parse_details(self, details):
        if details is None:
            return

        calls = []
        for _, detail in details.items():
            in_request = False
            in_response = False
            current_call = {}
            for line in detail.as_text().split("\n"):
                url_match = self.url_re.match(line)
                request_match = self.request_re.match(line)
                response_match = self.response_re.match(line)
                body_match = self.body_re.match(line)

                if url_match is not None:
                    if current_call != {}:
                        calls.append(current_call.copy())
                        current_call = {}
                        in_request, in_response = False, False
                    current_call.update({
                        "name": url_match.group("name"),
                        "verb": url_match.group("verb"),
                        "status_code": url_match.group("code"),
                        "service": self.get_service(url_match.group("url")),
                        "url": self.url_path(url_match.group("url"))})
                elif request_match is not None:
                    in_request, in_response = True, False
                    current_call.update(
                        {"request_headers": request_match.group("headers")})
                elif in_request and body_match is not None:
                    in_request = False
                    current_call.update(
                        {"request_body": body_match.group(
                            "body")})
                elif response_match is not None:
                    in_request, in_response = False, True
                    current_call.update(
                        {"response_headers": response_match.group(
                            "headers")})
                elif in_response and body_match is not None:
                    in_response = False
                    current_call.update(
                        {"response_body": body_match.group("body")})
            if current_call != {}:
                calls.append(current_call.copy())

        return calls

    def get_service(self, url):
        match = self.port_re.match(url)
        if match is not None:
            return self.services.get(match.group("port"), "Unknown")
        return "Unknown"

    def url_path(self, url):
        match = self.path_re.match(url)
        if match is not None:
            path = match.group("path")
            path = self.uuid_re.sub(r'\1<uuid>\2', path)
            path = self.ip_re.sub(r'\1<ip>\2', path)
            path = self.id_re.sub(r'\1<id>\2', path)
            return path
        return url


class FileAccumulator(testtools.StreamResult):

    def __init__(self, non_subunit_name='pythonlogging'):
        super(FileAccumulator, self).__init__()
        self.route_codes = collections.defaultdict(io.BytesIO)
        self.non_subunit_name = non_subunit_name

    def status(self, **kwargs):
        if kwargs.get('file_name') != self.non_subunit_name:
            return
        file_bytes = kwargs.get('file_bytes')
        if not file_bytes:
            return
        route_code = kwargs.get('route_code')
        stream = self.route_codes[route_code]
        stream.write(file_bytes)


class ArgumentParser(argparse.ArgumentParser):

    def __init__(self):
        desc = DESCRIPTION
        super(ArgumentParser, self).__init__(description=desc)
        self.prog = "subunit-describe-calls"
        _parser_add_args(self)


def parse(stream, non_subunit_name, ports):
    if ports is not None and os.path.exists(ports):
        ports = json.loads(open(ports).read())

    url_parser = UrlParser(ports)
    suite = subunit.ByteStreamToStreamResult(
        stream, non_subunit_name=non_subunit_name)
    result = testtools.StreamToExtendedDecorator(url_parser)
    accumulator = FileAccumulator(non_subunit_name)
    result = testtools.StreamResultRouter(result)
    result.add_rule(accumulator, 'test_id', test_id=None)
    result.startTestRun()
    suite.run(result)

    for bytes_io in accumulator.route_codes.values():  # v1 processing
        bytes_io.seek(0)
        suite = subunit.ProtocolTestCase(bytes_io)
        suite.run(url_parser)
    result.stopTestRun()

    return url_parser


def output(url_parser, output_file, all_stdout):
    if output_file is not None:
        with open(output_file, "w") as outfile:
            outfile.write(json.dumps(url_parser.test_logs))
        return

    for test_name in url_parser.test_logs:
        items = url_parser.test_logs[test_name]
        sys.stdout.write('{0}\n'.format(test_name))
        if not items:
            sys.stdout.write('\n')
            continue
        for item in items:
            sys.stdout.write('\t- {0} {1} request for {2} to {3}\n'.format(
                item.get('status_code'), item.get('verb'),
                item.get('service'), item.get('url')))
            if all_stdout:
                sys.stdout.write('\t\t- request headers: {0}\n'.format(
                    item.get('request_headers')))
                sys.stdout.write('\t\t- request body: {0}\n'.format(
                    item.get('request_body')))
                sys.stdout.write('\t\t- response headers: {0}\n'.format(
                    item.get('response_headers')))
                sys.stdout.write('\t\t- response body: {0}\n'.format(
                    item.get('response_body')))
        sys.stdout.write('\n')


def entry_point(cl_args=None):
    print('Running subunit_describe_calls ...')
    if not cl_args:
        print("Use of: 'subunit-describe-calls' is deprecated, "
              "please use: 'tempest subunit-describe-calls'")
        cl_args = ArgumentParser().parse_args()
    parser = parse(cl_args.subunit, cl_args.non_subunit_name, cl_args.ports)
    output(parser, cl_args.output_file, cl_args.all_stdout)


def _parser_add_args(parser):
    parser.add_argument(
        "-s", "--subunit", metavar="<subunit file>",
        nargs="?", type=argparse.FileType('rb'), default=sys.stdin,
        help="The path to the subunit output file(default:stdin v1/v2 stream)"
    )

    parser.add_argument(
        "-n", "--non-subunit-name", metavar="<non subunit name>",
        default="pythonlogging",
        help="The name used in subunit to describe the file contents."
    )

    parser.add_argument(
        "-o", "--output-file", metavar="<output file>", default=None,
        help="The output file name for the json."
    )

    parser.add_argument(
        "-p", "--ports", metavar="<ports file>", default=None,
        help="A JSON file describing the ports for each service."
    )

    group = parser.add_mutually_exclusive_group()
    # the -v and --verbose command are for the old subunit-describe-calls
    # main() CLI interface.  It does not work with the new
    # tempest subunit-describe-callss CLI. So when the main CLI approach is
    # deleted this argument is not needed.
    group.add_argument(
        "-v", "--verbose", action='store_true', dest='all_stdout',
        help='Add Request and Response header and body data to stdout print.'
             ' NOTE: This argument deprecated and does not work with'
             ' tempest subunit-describe-calls CLI.'
             ' Use new option: "-a", "--all-stdout"'
    )
    group.add_argument(
        "-a", "--all-stdout", action='store_true',
        help="Add Request and Response header and body data to stdout print."
             " Note: this argument work with the subunit-describe-calls and"
             " tempest subunit-describe-calls CLI commands."
    )


class TempestSubunitDescribeCalls(Command):

    def get_parser(self, prog_name):
        parser = super(TempestSubunitDescribeCalls, self).get_parser(prog_name)
        _parser_add_args(parser)
        return parser

    def take_action(self, parsed_args):
        try:
            entry_point(parsed_args)

        except Exception:
            traceback.print_exc()
            raise

    def get_description(self):
        return DESCRIPTION


if __name__ == "__main__":
    entry_point()



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.network import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class TagsTest(base.BaseNetworkTest):
    """Tests the following operations in the tags API:

        Update all tags.
        Delete all tags.
        Check tag existence.
        Create a tag.
        List tags.
        Remove a tag.

    v2.0 of the Neutron API is assumed. The tag extension allows users to set
    tags on their networks. The extension supports networks only.
    """

    @classmethod
    def skip_checks(cls):
        super(TagsTest, cls).skip_checks()
        if not utils.is_extension_enabled('tag', 'network'):
            msg = "tag extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(TagsTest, cls).resource_setup()
        cls.network = cls.create_network()

    @decorators.idempotent_id('ee76bfaf-ac94-4d74-9ecc-4bbd4c583cb1')
    def test_create_list_show_update_delete_tags(self):
        """Validate that creating a tag on a network resource works"""
        tag_name = data_utils.rand_name(self.__class__.__name__ + '-Tag')
        self.tags_client.create_tag('networks', self.network['id'], tag_name)
        self.addCleanup(self.tags_client.delete_all_tags, 'networks',
                        self.network['id'])
        self.tags_client.check_tag_existence('networks', self.network['id'],
                                             tag_name)

        # Validate that listing tags on a network resource works.
        retrieved_tags = self.tags_client.list_tags(
            'networks', self.network['id'])['tags']
        self.assertEqual([tag_name], retrieved_tags)

        # Generate 3 new tag names.
        replace_tags = [data_utils.rand_name(
            self.__class__.__name__ + '-Tag') for _ in range(3)]

        # Replace the current tag with the 3 new tags and validate that the
        # network resource has the 3 new tags.
        updated_tags = self.tags_client.update_all_tags(
            'networks', self.network['id'], replace_tags)['tags']
        self.assertEqual(3, len(updated_tags))
        self.assertEqual(set(replace_tags), set(updated_tags))

        # Delete the first tag and check that it has been removed.
        self.tags_client.delete_tag(
            'networks', self.network['id'], replace_tags[0])
        self.assertRaises(lib_exc.NotFound,
                          self.tags_client.check_tag_existence, 'networks',
                          self.network['id'], replace_tags[0])
        for i in range(1, 3):
            self.tags_client.check_tag_existence(
                'networks', self.network['id'], replace_tags[i])

        # Delete all the remaining tags and check that they have been removed.
        self.tags_client.delete_all_tags('networks', self.network['id'])
        for i in range(1, 3):
            self.assertRaises(lib_exc.NotFound,
                              self.tags_client.check_tag_existence, 'networks',
                              self.network['id'], replace_tags[i])


class TagsExtTest(base.BaseNetworkTest):
    """Tests the following operations in the tags API:

        Update all tags.
        Delete all tags.
        Check tag existence.
        Create a tag.
        List tags.
        Remove a tag.

    v2.0 of the Neutron API is assumed. The tag-ext or standard-attr-tag
    extension allows users to set tags on the following resources: subnets,
    ports, routers and subnetpools.
    from stein release the tag-ext has been renamed to standard-attr-tag
    """

    # NOTE(felipemonteiro): The supported resource names are plural. Use
    # the singular case for the corresponding class resource object.
    SUPPORTED_RESOURCES = ['subnets', 'ports', 'routers', 'subnetpools']

    @classmethod
    def skip_checks(cls):
        super(TagsExtTest, cls).skip_checks()
        # Added condition to support backward compatiblity since
        # tag-ext has been renamed to standard-attr-tag
        if not (utils.is_extension_enabled('tag-ext', 'network') or
                utils.is_extension_enabled('standard-attr-tag', 'network')):
            msg = ("neither tag-ext nor standard-attr-tag extensions "
                   "are enabled.")
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(TagsExtTest, cls).resource_setup()
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)
        cls.port = cls.create_port(cls.network)
        cls.router = cls.create_router()

        subnetpool_name = data_utils.rand_name(cls.__name__ + '-Subnetpool')
        prefix = CONF.network.default_network
        cls.subnetpool = cls.subnetpools_client.create_subnetpool(
            name=subnetpool_name, prefixes=prefix)['subnetpool']
        cls.addClassResourceCleanup(cls.subnetpools_client.delete_subnetpool,
                                    cls.subnetpool['id'])

    def _create_tags_for_each_resource(self):
        # Create a tag for each resource in `SUPPORTED_RESOURCES` and return
        # the list of tag names.
        tag_names = []

        for resource in self.SUPPORTED_RESOURCES:
            tag_name = data_utils.rand_name(self.__class__.__name__ + '-Tag')
            tag_names.append(tag_name)
            resource_object = getattr(self, resource[:-1])

            self.tags_client.create_tag(resource, resource_object['id'],
                                        tag_name)
            self.addCleanup(self.tags_client.delete_all_tags, resource,
                            resource_object['id'])

        return tag_names

    @decorators.idempotent_id('c6231efa-9a89-4adf-b050-2a3156b8a1d9')
    def test_create_check_list_and_delete_tags(self):
        """Test tag operations on subnets/ports/routers/subnetpools"""
        tag_names = self._create_tags_for_each_resource()

        for i, resource in enumerate(self.SUPPORTED_RESOURCES):
            # Ensure that a tag was created for each resource.
            resource_object = getattr(self, resource[:-1])
            retrieved_tags = self.tags_client.list_tags(
                resource, resource_object['id'])['tags']
            self.assertEqual(1, len(retrieved_tags))
            self.assertEqual(tag_names[i], retrieved_tags[0])

            # Check that the expected tag exists for each resource.
            self.tags_client.check_tag_existence(
                resource, resource_object['id'], tag_names[i])

            # Delete the tag and ensure it was deleted.
            self.tags_client.delete_tag(
                resource, resource_object['id'], tag_names[i])
            retrieved_tags = self.tags_client.list_tags(
                resource, resource_object['id'])['tags']
            self.assertEmpty(retrieved_tags)

    @decorators.idempotent_id('663a90f5-f334-4b44-afe0-c5fc1d408791')
    def test_update_and_delete_all_tags(self):
        """Test update/delete all tags on subnets/ports/routers/subnetpools"""
        self._create_tags_for_each_resource()

        for resource in self.SUPPORTED_RESOURCES:
            # Generate 3 new tag names.
            replace_tags = [data_utils.rand_name(
                self.__class__.__name__ + '-Tag') for _ in range(3)]

            # Replace the current tag with the 3 new tags and validate that the
            # current resource has the 3 new tags.
            resource_object = getattr(self, resource[:-1])
            updated_tags = self.tags_client.update_all_tags(
                resource, resource_object['id'], replace_tags)['tags']
            self.assertEqual(3, len(updated_tags))
            self.assertEqual(set(replace_tags), set(updated_tags))

            # Delete all the tags and check that they have been removed.
            self.tags_client.delete_all_tags(resource, resource_object['id'])
            for i in range(3):
                self.assertRaises(
                    lib_exc.NotFound, self.tags_client.check_tag_existence,
                    resource, resource_object['id'], replace_tags[i])



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF


class VolumeManageAdminTest(base.BaseVolumeAdminTest):
    """Test volume manage by admin users"""

    @classmethod
    def skip_checks(cls):
        super(VolumeManageAdminTest, cls).skip_checks()

        if not CONF.volume_feature_enabled.manage_volume:
            raise cls.skipException("Manage volume tests are disabled")

        if len(CONF.volume.manage_volume_ref) != 2:
            msg = ("Manage volume ref is not correctly configured, "
                   "it should be a list of two elements")
            raise exceptions.InvalidConfiguration(msg)

    @decorators.idempotent_id('70076c71-0ce1-4208-a8ff-36a66e65cc1e')
    def test_unmanage_manage_volume(self):
        """Test unmanaging and managing volume"""
        # Create original volume
        org_vol_id = self.create_volume()['id']
        org_vol_info = self.admin_volume_client.show_volume(
            org_vol_id)['volume']

        # Unmanage the original volume
        self.admin_volume_client.unmanage_volume(org_vol_id)
        self.admin_volume_client.wait_for_resource_deletion(org_vol_id)

        # Verify the original volume does not exist in volume list
        params = {'all_tenants': 1}
        all_tenants_volumes = self.admin_volume_client.list_volumes(
            detail=True, params=params)['volumes']
        self.assertNotIn(org_vol_id, [v['id'] for v in all_tenants_volumes])

        # Manage volume
        new_vol_name = data_utils.rand_name(
            self.__class__.__name__ + '-volume')
        new_vol_ref = {
            'name': new_vol_name,
            'host': org_vol_info['os-vol-host-attr:host'],
            'ref': {CONF.volume.manage_volume_ref[0]:
                    CONF.volume.manage_volume_ref[1] % org_vol_id},
            'volume_type': org_vol_info['volume_type'],
            'availability_zone': org_vol_info['availability_zone']}
        new_vol_id = self.admin_volume_manage_client.manage_volume(
            **new_vol_ref)['volume']['id']
        self.addCleanup(self.delete_volume,
                        self.admin_volume_client, new_vol_id)
        waiters.wait_for_volume_resource_status(self.admin_volume_client,
                                                new_vol_id, 'available')

        # Compare the managed volume with the original
        new_vol_info = self.admin_volume_client.show_volume(
            new_vol_id)['volume']
        self.assertNotIn(new_vol_id, [org_vol_id])
        self.assertEqual(new_vol_info['name'], new_vol_name)
        for key in ['size',
                    'volume_type',
                    'availability_zone',
                    'os-vol-host-attr:host']:
            self.assertEqual(new_vol_info[key], org_vol_info[key])



# Copyright 2016 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import image
from tempest.lib.common import rest_client
from tempest.tests import base


class TestImage(base.TestCase):

    def test_get_image_meta_from_headers(self):
        resp = {
            'x-image-meta-id': 'ea30c926-0629-4400-bb6e-f8a8da6a4e56',
            'x-image-meta-owner': '8f421f9470e645b1b10f5d2db7804924',
            'x-image-meta-status': 'queued',
            'x-image-meta-name': 'New Http Image'
        }
        respbody = rest_client.ResponseBody(resp)
        observed = image.get_image_meta_from_headers(respbody)

        expected = {
            'properties': {},
            'id': 'ea30c926-0629-4400-bb6e-f8a8da6a4e56',
            'owner': '8f421f9470e645b1b10f5d2db7804924',
            'status': 'queued',
            'name': 'New Http Image'
        }
        self.assertEqual(expected, observed)

    def test_image_meta_to_headers(self):
        observed = image.image_meta_to_headers(
            name='test',
            container_format='wrong',
            disk_format='vhd',
            copy_from='http://localhost/images/10',
            properties={'foo': 'bar'},
            api={'abc': 'def'},
            purge_props=True)

        expected = {
            'x-image-meta-name': 'test',
            'x-image-meta-container_format': 'wrong',
            'x-image-meta-disk_format': 'vhd',
            'x-glance-api-copy-from': 'http://localhost/images/10',
            'x-image-meta-property-foo': 'bar',
            'x-glance-api-property-abc': 'def',
            'x-glance-registry-purge-props': True
        }
        self.assertEqual(expected, observed)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.object_storage import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class ObjectTestACLs(base.BaseObjectTest):
    """Test object ACLs"""

    credentials = [['operator', CONF.object_storage.operator_role],
                   ['operator_alt', CONF.object_storage.operator_role]]

    def setUp(self):
        super(ObjectTestACLs, self).setUp()
        self.container_name = self.create_container()

    def tearDown(self):
        self.delete_containers()
        super(ObjectTestACLs, self).tearDown()

    @decorators.idempotent_id('a3270f3f-7640-4944-8448-c7ea783ea5b6')
    def test_read_object_with_rights(self):
        """Test reading object using authorized user"""
        # update X-Container-Read metadata ACL
        tenant_id = self.os_roles_operator_alt.credentials.tenant_id
        user_id = self.os_roles_operator_alt.credentials.user_id
        cont_headers = {'X-Container-Read': tenant_id + ':' + user_id}
        container_client = self.os_roles_operator.container_client
        resp_meta, _ = (
            container_client.create_update_or_delete_container_metadata(
                self.container_name, create_update_metadata=cont_headers,
                create_update_metadata_prefix=''))
        self.assertHeaders(resp_meta, 'Container', 'POST')
        # create object
        object_name = data_utils.rand_name(name='Object')
        resp, _ = self.os_roles_operator.object_client.create_object(
            self.container_name, object_name, 'data')
        self.assertHeaders(resp, 'Object', 'PUT')
        # set alternative authentication data; cannot simply use the
        # other object client.
        self.os_roles_operator.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=self.os_roles_operator_alt.object_client.auth_provider.
            auth_data)
        resp, _ = self.os_roles_operator.object_client.get_object(
            self.container_name, object_name)
        self.assertHeaders(resp, 'Object', 'GET')

    @decorators.idempotent_id('aa58bfa5-40d9-4bc3-82b4-d07f4a9e392a')
    def test_write_object_with_rights(self):
        """Test writing object using authorized user"""
        # update X-Container-Write metadata ACL
        tenant_id = self.os_roles_operator_alt.credentials.tenant_id
        user_id = self.os_roles_operator_alt.credentials.user_id
        cont_headers = {'X-Container-Write': tenant_id + ':' + user_id}
        container_client = self.os_roles_operator.container_client
        resp_meta, _ = (
            container_client.create_update_or_delete_container_metadata(
                self.container_name, create_update_metadata=cont_headers,
                create_update_metadata_prefix=''))
        self.assertHeaders(resp_meta, 'Container', 'POST')
        # set alternative authentication data; cannot simply use the
        # other object client.
        self.os_roles_operator.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=self.os_roles_operator_alt.object_client.auth_provider.
            auth_data)
        # Trying to write the object with rights
        object_name = data_utils.rand_name(name='Object')
        resp, _ = self.os_roles_operator.object_client.create_object(
            self.container_name,
            object_name, 'data', headers={})
        self.assertHeaders(resp, 'Object', 'PUT')



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.network import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class RoutersAdminNegativeTest(base.BaseAdminNetworkTest):
    """Admin negative tests of routers"""

    @classmethod
    def skip_checks(cls):
        super(RoutersAdminNegativeTest, cls).skip_checks()
        if not utils.is_extension_enabled('router', 'network'):
            msg = "router extension not enabled."
            raise cls.skipException(msg)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7101cc02-058a-11e7-93e1-fa163e4fa634')
    @utils.requires_ext(extension='ext-gw-mode', service='network')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    def test_router_set_gateway_used_ip_returns_409(self):
        """Test creating router with gateway set to used ip should fail"""
        # At first create a address from public_network_id
        port = self.admin_ports_client.create_port(
            name=data_utils.rand_name(self.__class__.__name__),
            network_id=CONF.network.public_network_id)['port']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_ports_client.delete_port,
                        port_id=port['id'])
        # Add used ip and subnet_id in external_fixed_ips
        fixed_ip = {
            'subnet_id': port['fixed_ips'][0]['subnet_id'],
            'ip_address': port['fixed_ips'][0]['ip_address']
        }
        external_gateway_info = {
            'network_id': CONF.network.public_network_id,
            'external_fixed_ips': [fixed_ip]
        }
        # Create a router and set gateway to used ip
        self.assertRaises(lib_exc.Conflict,
                          self.admin_routers_client.create_router,
                          external_gateway_info=external_gateway_info)


class RoutersAdminNegativeIpV6Test(RoutersAdminNegativeTest):
    _ip_version = 6



# Copyright 2015 OpenStack Foundation.
# Copyright 2015, Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest import config
from tempest.lib import decorators


CONF = config.CONF


class TestApiDiscovery(base.BaseIdentityV3Test):
    """Tests for identity API discovery features."""

    @decorators.idempotent_id('79aec9ae-710f-4c54-a4fc-3aa25b4feac3')
    def test_identity_v3_existence(self):
        """Test that identity v3 version should exist"""
        versions = self.non_admin_versions_client.list_versions()
        found = any(
            "v3" in version.get('id')
            for version in versions['versions']['values'])
        self.assertEqual(CONF.identity_feature_enabled.api_v3, found)

    @decorators.idempotent_id('721f480f-35b6-46c7-846e-047e6acea0dc')
    @decorators.attr(type='smoke')
    def test_list_api_versions(self):
        """Test listing identity api versions

        NOTE: Actually this API doesn't depend on v3 API at all, because
        the API operation is "GET /" without v3's endpoint. The reason of
        this test path is just v3 API is CURRENT on Keystone side.
        """
        versions = self.non_admin_versions_client.list_versions()
        expected_resources = ('id', 'links', 'media-types', 'status',
                              'updated')

        for version in versions['versions']["values"]:
            for res in expected_resources:
                self.assertIn(res, version)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('b9232f5e-d9e5-4d97-b96c-28d3db4de1bd')
    def test_api_version_resources(self):
        """Test showing identity v3 api version resources"""
        descr = self.non_admin_client.show_api_description()['version']
        expected_resources = ('id', 'links', 'media-types', 'status',
                              'updated')

        keys = descr.keys()
        for res in expected_resources:
            self.assertIn(res, keys)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('657c1970-4722-4189-8831-7325f3bc4265')
    def test_api_media_types(self):
        """Test showing identity v3 api version media type"""
        descr = self.non_admin_client.show_api_description()['version']
        # Get MIME type bases and descriptions
        media_types = [(media_type['base'], media_type['type']) for
                       media_type in descr['media-types']]
        # These are supported for API version 2
        supported_types = [('application/json',
                            'application/vnd.openstack.identity-v3+json')]

        # Check if supported types exist in response body
        for s_type in supported_types:
            self.assertIn(s_type, media_types)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('8879a470-abfb-47bb-bb8d-5a7fd279ad1e')
    def test_api_version_statuses(self):
        """Test showing identity v3 api version status"""
        descr = self.non_admin_client.show_api_description()['version']
        status = descr['status'].lower()
        supported_statuses = ['current', 'stable', 'experimental',
                              'supported', 'deprecated']

        self.assertIn(status, supported_statuses)



# Copyright 2014 IBM Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import quotas

# NOTE(mriedem): os-quota-class-sets responses are the same as os-quota-sets
# except for the key in the response body is quota_class_set instead of
# quota_set, so update this copy of the schema from os-quota-sets.
get_quota_class_set = copy.deepcopy(quotas.get_quota_set)
get_quota_class_set['response_body']['properties']['quota_class_set'] = (
    get_quota_class_set['response_body']['properties'].pop('quota_set'))
get_quota_class_set['response_body']['required'] = ['quota_class_set']

update_quota_class_set = copy.deepcopy(quotas.update_quota_set)
update_quota_class_set['response_body']['properties']['quota_class_set'] = (
    update_quota_class_set['response_body']['properties'].pop('quota_set'))
update_quota_class_set['response_body']['required'] = ['quota_class_set']



# Copyright 2019 SUSE LLC
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class AccessRulesV3Test(base.BaseIdentityV3Test):

    @classmethod
    def skip_checks(cls):
        super(AccessRulesV3Test, cls).skip_checks()
        if not CONF.identity_feature_enabled.access_rules:
            raise cls.skipException("Application credential access rules are "
                                    "not available in this environment")

    @classmethod
    def resource_setup(cls):
        super(AccessRulesV3Test, cls).resource_setup()
        cls.user_id = cls.os_primary.credentials.user_id
        cls.project_id = cls.os_primary.credentials.project_id

    def setUp(self):
        super(AccessRulesV3Test, self).setUp()
        ac = self.non_admin_app_creds_client
        access_rules = [
            {
                "path": "/v2.1/servers/*/ips",
                "method": "GET",
                "service": "compute"
            }
        ]
        self.app_cred = ac.create_application_credential(
            self.user_id,
            name=data_utils.rand_name('application_credential'),
            access_rules=access_rules
        )['application_credential']

    @decorators.idempotent_id('2354c498-5119-4ba5-9f0d-44f16f78fb0e')
    def test_list_access_rules(self):
        ar = self.non_admin_access_rules_client.list_access_rules(self.user_id)
        self.assertEqual(1, len(ar['access_rules']))

    @decorators.idempotent_id('795dd507-ca1e-40e9-ba90-ff0a08689ba4')
    def test_show_access_rule(self):
        access_rule_id = self.app_cred['access_rules'][0]['id']
        self.non_admin_access_rules_client.show_access_rule(
            self.user_id, access_rule_id)

    @decorators.idempotent_id('278757e9-e193-4bf8-adf2-0b0a229a17d0')
    def test_delete_access_rule(self):
        access_rule_id = self.app_cred['access_rules'][0]['id']
        app_cred_id = self.app_cred['id']
        self.assertRaises(
            lib_exc.Forbidden,
            self.non_admin_access_rules_client.delete_access_rule,
            self.user_id,
            access_rule_id)
        self.non_admin_app_creds_client.delete_application_credential(
            self.user_id, app_cred_id)
        ar = self.non_admin_access_rules_client.list_access_rules(self.user_id)
        self.assertEqual(1, len(ar['access_rules']))
        self.non_admin_access_rules_client.delete_access_rule(
            self.user_id, access_rule_id)
        ar = self.non_admin_access_rules_client.list_access_rules(self.user_id)
        self.assertEqual(0, len(ar['access_rules']))



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import agents_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestAgentsClient(base.BaseServiceTest):
    FAKE_CREATE_AGENT = {
        "agent": {
            "url": "http://foo.com",
            "hypervisor": "kvm",
            "md5hash": "md5",
            "version": "2",
            "architecture": "x86_64",
            "os": "linux",
            "agent_id": 1
        }
    }

    FAKE_UPDATE_AGENT = {
        "agent": {
            "url": "http://foo.com",
            "md5hash": "md5",
            "version": "2",
            "agent_id": 1
        }
    }

    def setUp(self):
        super(TestAgentsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = agents_client.AgentsClient(fake_auth,
                                                 'compute', 'regionOne')

    def _test_list_agents(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_agents,
            'tempest.lib.common.rest_client.RestClient.get',
            {"agents": []},
            bytes_body)
        self.check_service_client_function(
            self.client.list_agents,
            'tempest.lib.common.rest_client.RestClient.get',
            {"agents": []},
            bytes_body,
            hypervisor="kvm")

    def _test_create_agent(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_agent,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_AGENT,
            bytes_body,
            url="http://foo.com", hypervisor="kvm", md5hash="md5",
            version="2", architecture="x86_64", os="linux")

    def _test_delete_agent(self):
        self.check_service_client_function(
            self.client.delete_agent,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, agent_id="1")

    def _test_update_agent(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_agent,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_AGENT,
            bytes_body,
            agent_id="1", url="http://foo.com", md5hash="md5", version="2")

    def test_list_agents_with_str_body(self):
        self._test_list_agents()

    def test_list_agents_with_bytes_body(self):
        self._test_list_agents(bytes_body=True)

    def test_create_agent_with_str_body(self):
        self._test_create_agent()

    def test_create_agent_with_bytes_body(self):
        self._test_create_agent(bytes_body=True)

    def test_delete_agent(self):
        self._test_delete_agent()

    def test_update_agent_with_str_body(self):
        self._test_update_agent()

    def test_update_agent_with_bytes_body(self):
        self._test_update_agent(bytes_body=True)



# Copyright 2017 AT&T Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.network import floating_ips_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestFloatingIPsClient(base.BaseServiceTest):

    FAKE_FLOATING_IPS = {
        "floatingips": [
            {
                "router_id": "d23abc8d-2991-4a55-ba98-2aaea84cc72f",
                "description": "for test",
                "dns_domain": "my-domain.org.",
                "dns_name": "myfip",
                "created_at": "2016-12-21T10:55:50Z",
                "updated_at": "2016-12-21T10:55:53Z",
                "revision_number": 1,
                "project_id": "4969c491a3c74ee4af974e6d800c62de",
                "tenant_id": "4969c491a3c74ee4af974e6d800c62de",
                "floating_network_id": "376da547-b977-4cfe-9cba-275c80debf57",
                "fixed_ip_address": "10.0.0.3",
                "floating_ip_address": "172.24.4.228",
                "port_id": "ce705c24-c1ef-408a-bda3-7bbd946164ab",
                "id": "2f245a7b-796b-4f26-9cf9-9e82d248fda7",
                "status": "ACTIVE",
                "port_details": {
                    "status": "ACTIVE",
                    "name": "",
                    "admin_state_up": True,
                    "network_id": "02dd8479-ef26-4398-a102-d19d0a7b3a1f",
                    "device_owner": "compute:nova",
                    "mac_address": "fa:16:3e:b1:3b:30",
                    "device_id": "8e3941b4-a6e9-499f-a1ac-2a4662025cba"
                },
                "tags": ["tag1,tag2"],
                "port_forwardings": []
            },
            {
                "router_id": None,
                "description": "for test",
                "dns_domain": "my-domain.org.",
                "dns_name": "myfip2",
                "created_at": "2016-12-21T11:55:50Z",
                "updated_at": "2016-12-21T11:55:53Z",
                "revision_number": 2,
                "project_id": "4969c491a3c74ee4af974e6d800c62de",
                "tenant_id": "4969c491a3c74ee4af974e6d800c62de",
                "floating_network_id": "376da547-b977-4cfe-9cba-275c80debf57",
                "fixed_ip_address": None,
                "floating_ip_address": "172.24.4.227",
                "port_id": None,
                "id": "61cea855-49cb-4846-997d-801b70c71bdd",
                "status": "DOWN",
                "port_details": None,
                "tags": ["tag1,tag2"],
                "port_forwardings": []
            }
        ]
    }

    FAKE_FLOATING_IP_ID = "2f245a7b-796b-4f26-9cf9-9e82d248fda7"

    def setUp(self):
        super(TestFloatingIPsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.floating_ips_client = floating_ips_client.FloatingIPsClient(
            fake_auth, "compute", "regionOne")

    def _test_list_floatingips(self, bytes_body=False):
        self.check_service_client_function(
            self.floating_ips_client.list_floatingips,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_FLOATING_IPS,
            bytes_body,
            200)

    def _test_create_floatingip(self, bytes_body=False):
        self.check_service_client_function(
            self.floating_ips_client.create_floatingip,
            "tempest.lib.common.rest_client.RestClient.post",
            {"floatingip": self.FAKE_FLOATING_IPS["floatingips"][1]},
            bytes_body,
            201,
            floating_network_id="172.24.4.228")

    def _test_show_floatingip(self, bytes_body=False):
        self.check_service_client_function(
            self.floating_ips_client.show_floatingip,
            "tempest.lib.common.rest_client.RestClient.get",
            {"floatingip": self.FAKE_FLOATING_IPS["floatingips"][0]},
            bytes_body,
            200,
            floatingip_id=self.FAKE_FLOATING_IP_ID)

    def _test_update_floatingip(self, bytes_body=False):
        update_kwargs = {
            "port_id": "fc861431-0e6c-4842-a0ed-e2363f9bc3a8"
        }

        resp_body = {
            "floatingip": copy.deepcopy(
                self.FAKE_FLOATING_IPS["floatingips"][0]
            )
        }
        resp_body["floatingip"].update(update_kwargs)

        self.check_service_client_function(
            self.floating_ips_client.update_floatingip,
            "tempest.lib.common.rest_client.RestClient.put",
            resp_body,
            bytes_body,
            200,
            floatingip_id=self.FAKE_FLOATING_IP_ID,
            **update_kwargs)

    def test_list_floatingips_with_str_body(self):
        self._test_list_floatingips()

    def test_list_floatingips_with_bytes_body(self):
        self._test_list_floatingips(bytes_body=True)

    def test_create_floatingip_with_str_body(self):
        self._test_create_floatingip()

    def test_create_floatingip_with_bytes_body(self):
        self._test_create_floatingip(bytes_body=True)

    def test_show_floatingips_with_str_body(self):
        self._test_show_floatingip()

    def test_show_floatingips_with_bytes_body(self):
        self._test_show_floatingip(bytes_body=True)

    def test_update_floatingip_with_str_body(self):
        self._test_update_floatingip()

    def test_update_floatingip_with_bytes_body(self):
        self._test_update_floatingip(bytes_body=True)

    def test_delete_floatingip(self):
        self.check_service_client_function(
            self.floating_ips_client.delete_floatingip,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            floatingip_id=self.FAKE_FLOATING_IP_ID)



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures
from oslo_config import cfg
import testtools

from tempest.common import utils
from tempest import config
from tempest import exceptions
from tempest import test
from tempest.tests import base
from tempest.tests import fake_config


class BaseDecoratorsTest(base.TestCase):
    def setUp(self):
        super(BaseDecoratorsTest, self).setUp()
        self.config_fixture = self.useFixture(fake_config.ConfigFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)


class TestServicesDecorator(BaseDecoratorsTest):
    def _test_services_helper(self, *decorator_args):
        class TestFoo(test.BaseTestCase):
            @utils.services(*decorator_args)
            def test_bar(self):
                return 0

        t = TestFoo('test_bar')
        self.assertEqual(set(decorator_args), getattr(t.test_bar,
                                                      '__testtools_attrs'))
        self.assertEqual(t.test_bar(), 0)

    def test_services_decorator_with_single_service(self):
        self._test_services_helper('compute')

    def test_services_decorator_with_multiple_services(self):
        self._test_services_helper('compute', 'network')

    def test_services_decorator_with_duplicated_service(self):
        self._test_services_helper('compute', 'compute')

    def test_services_decorator_with_invalid_service(self):
        self.assertRaises(exceptions.InvalidServiceTag,
                          self._test_services_helper, 'compute',
                          'bad_service')

    def test_services_decorator_with_service_valid_and_unavailable(self):
        self.useFixture(fixtures.MockPatchObject(test.CONF.service_available,
                                                 'cinder', False))
        self.assertRaises(testtools.TestCase.skipException,
                          self._test_services_helper, 'compute',
                          'volume')

    def test_services_list(self):
        service_list = utils.get_service_list()
        for service in service_list:
            try:
                self._test_services_helper(service)
            except exceptions.InvalidServiceTag:
                self.fail('%s is not listed in the valid service tag list'
                          % service)
            except KeyError:
                # NOTE(mtreinish): This condition is to test for an entry in
                # the outer decorator list but not in the service_list dict.
                # However, because we're looping over the service_list dict
                # it's unlikely we'll trigger this. So manual review is still
                # need for the list in the outer decorator.
                self.fail('%s is in the list of valid service tags but there '
                          'is no corresponding entry in the dict returned from'
                          ' get_service_list()' % service)
            except testtools.TestCase.skipException:
                # Test didn't raise an exception because of an incorrect list
                # entry so move onto the next entry
                continue


class TestRequiresExtDecorator(BaseDecoratorsTest):
    def setUp(self):
        super(TestRequiresExtDecorator, self).setUp()
        cfg.CONF.set_default('api_extensions', ['enabled_ext', 'another_ext'],
                             'compute-feature-enabled')

    def _test_requires_ext_helper(self, expected_to_skip=True,
                                  **decorator_args):
        class TestFoo(test.BaseTestCase):
            @utils.requires_ext(**decorator_args)
            def test_bar(self):
                return 0

        t = TestFoo('test_bar')
        if expected_to_skip:
            self.assertRaises(testtools.TestCase.skipException, t.test_bar)
        else:
            try:
                self.assertEqual(t.test_bar(), 0)
            except testtools.TestCase.skipException:
                # We caught a skipException but we didn't expect to skip
                # this test so raise a hard test failure instead.
                raise testtools.TestCase.failureException(
                    "Not supposed to skip")

    def test_requires_ext_decorator(self):
        self._test_requires_ext_helper(expected_to_skip=False,
                                       extension='enabled_ext',
                                       service='compute')

    def test_requires_ext_decorator_disabled_ext(self):
        self._test_requires_ext_helper(extension='disabled_ext',
                                       service='compute')

    def test_requires_ext_decorator_with_all_ext_enabled(self):
        cfg.CONF.set_default('api_extensions', ['all'],
                             group='compute-feature-enabled')
        self._test_requires_ext_helper(expected_to_skip=False,
                                       extension='random_ext',
                                       service='compute')

    def test_requires_ext_decorator_bad_service(self):
        self.assertRaises(KeyError,
                          self._test_requires_ext_helper,
                          extension='enabled_ext',
                          service='bad_service')



# Copyright 2018 SUSE Linux GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import application_credentials_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestApplicationCredentialsClient(base.BaseServiceTest):
    FAKE_CREATE_APP_CRED = {
        "application_credential": {
            "name": "monitoring",
            "secret": "rEaqvJka48mpv",
            "description": "Application credential for monitoring.",
            "expires_at": "2018-02-27T18:30:59Z",
            "roles": [
                {"name": "Reader"}
            ],
            "access_rules": [
                {
                    "path": "/v2.0/metrics",
                    "method": "GET",
                    "service": "monitoring"
                }
            ],
            "unrestricted": False
        }
    }

    FAKE_LIST_APP_CREDS = {
        "links": {
            "self": "http://example.com/identity/v3/users/" +
                    "fd786d56402c4d1691372e7dee0d00b5/application_credentials",
            "previous": None,
            "next": None
        },
        "application_credentials": [
            {
                "description": "Application credential for backups.",
                "roles": [
                    {
                        "domain_id": None,
                        "name": "Writer",
                        "id": "6aff702516544aeca22817fd3bc39683"
                    }
                ],
                "access_rules": [
                ],
                "links": {
                    "self": "http://example.com/identity/v3/users/" +
                            "fd786d56402c4d1691372e7dee0d00b5/" +
                            "application_credentials/" +
                            "308a7e905eee4071aac5971744c061f6"
                },
                "expires_at": "2018-02-27T18:30:59.000000",
                "unrestricted": False,
                "project_id": "231c62fb0fbd485b995e8b060c3f0d98",
                "id": "308a7e905eee4071aac5971744c061f6",
                "name": "backups"
            },
            {
                "description": "Application credential for monitoring.",
                "roles": [
                    {
                        "id": "6aff702516544aeca22817fd3bc39683",
                        "domain_id": None,
                        "name": "Reader"
                    }
                ],
                "access_rules": [
                    {
                        "path": "/v2.0/metrics",
                        "id": "07d719df00f349ef8de77d542edf010c",
                        "service": "monitoring",
                        "method": "GET"
                    }
                ],
                "links": {
                    "self": "http://example.com/identity/v3/users/" +
                            "fd786d56402c4d1691372e7dee0d00b5/" +
                            "application_credentials/" +
                            "58d61ff8e6e34accb35874016d1dba8b"
                },
                "expires_at": "2018-02-27T18:30:59.000000",
                "unrestricted": False,
                "project_id": "231c62fb0fbd485b995e8b060c3f0d98",
                "id": "58d61ff8e6e34accb35874016d1dba8b",
                "name": "monitoring"
            }
        ]
    }

    FAKE_APP_CRED_INFO = {
        "application_credential": {
            "description": "Application credential for monitoring.",
            "roles": [
                {
                    "id": "6aff702516544aeca22817fd3bc39683",
                    "domain_id": None,
                    "name": "Reader"
                }
            ],
            "access_rules": [
                {
                    "path": "/v2.0/metrics",
                    "id": "07d719df00f349ef8de77d542edf010c",
                    "service": "monitoring",
                    "method": "GET"
                }
            ],
            "links": {
                "self": "http://example.com/identity/v3/users/" +
                        "fd786d56402c4d1691372e7dee0d00b5/" +
                        "application_credentials/" +
                        "58d61ff8e6e34accb35874016d1dba8b"
            },
            "expires_at": "2018-02-27T18:30:59.000000",
            "unrestricted": False,
            "project_id": "231c62fb0fbd485b995e8b060c3f0d98",
            "id": "58d61ff8e6e34accb35874016d1dba8b",
            "name": "monitoring"
        }
    }

    def setUp(self):
        super(TestApplicationCredentialsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = \
            application_credentials_client.ApplicationCredentialsClient(
                fake_auth, 'identity', 'regionOne')

    def _test_create_app_cred(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_application_credential,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_APP_CRED,
            bytes_body,
            status=201,
            user_id="123456")

    def _test_show_app_cred(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_application_credential,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_APP_CRED_INFO,
            bytes_body,
            user_id="123456",
            application_credential_id="58d61ff8e6e34accb35874016d1dba8b")

    def _test_list_app_creds(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_application_credentials,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_APP_CREDS,
            bytes_body,
            user_id="123456")

    def test_create_application_credential_with_str_body(self):
        self._test_create_app_cred()

    def test_create_application_credential_with_bytes_body(self):
        self._test_create_app_cred(bytes_body=True)

    def test_show_application_credential_with_str_body(self):
        self._test_show_app_cred()

    def test_show_application_credential_with_bytes_body(self):
        self._test_show_app_cred(bytes_body=True)

    def test_list_application_credential_with_str_body(self):
        self._test_list_app_creds()

    def test_list_application_credential_with_bytes_body(self):
        self._test_list_app_creds(bytes_body=True)

    def test_delete_application_credential(self):
        self.check_service_client_function(
            self.client.delete_application_credential,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            user_id="123456",
            application_credential_id="58d61ff8e6e34accb35874016d1dba8b",
            status=204)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF


class DomainsTestJSON(base.BaseIdentityV3AdminTest):
    """Test identity domains"""

    @classmethod
    def resource_setup(cls):
        super(DomainsTestJSON, cls).resource_setup()
        # Create some test domains to be used during tests
        # One of those domains will be disabled
        cls.setup_domains = list()
        for i in range(3):
            domain = cls.create_domain(enabled=i < 2)
            cls.setup_domains.append(domain)

    @decorators.idempotent_id('8cf516ef-2114-48f1-907b-d32726c734d4')
    def test_list_domains(self):
        """Test listing domains"""
        fetched_ids = list()
        # List and Verify Domains
        body = self.domains_client.list_domains()['domains']
        for d in body:
            fetched_ids.append(d['id'])
        missing_doms = [d for d in self.setup_domains
                        if d['id'] not in fetched_ids]
        self.assertEmpty(missing_doms)

    @decorators.idempotent_id('c6aee07b-4981-440c-bb0b-eb598f58ffe9')
    def test_list_domains_filter_by_name(self):
        """Test listing domains filtering by name"""
        params = {'name': self.setup_domains[0]['name']}
        fetched_domains = self.domains_client.list_domains(
            **params)['domains']
        # Verify the filtered list is correct, domain names are unique
        # so exactly one domain should be found with the provided name
        self.assertEqual(1, len(fetched_domains))
        self.assertEqual(self.setup_domains[0]['name'],
                         fetched_domains[0]['name'])

    @decorators.idempotent_id('3fd19840-65c1-43f8-b48c-51bdd066dff9')
    def test_list_domains_filter_by_enabled(self):
        """Test listing domains filtering by enabled domains"""
        params = {'enabled': True}
        fetched_domains = self.domains_client.list_domains(
            **params)['domains']
        # Verify the filtered list is correct
        self.assertIn(self.setup_domains[0], fetched_domains)
        self.assertIn(self.setup_domains[1], fetched_domains)
        for domain in fetched_domains:
            self.assertEqual(True, domain['enabled'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('f2f5b44a-82e8-4dad-8084-0661ea3b18cf')
    def test_create_update_delete_domain(self):
        """Test creating, updating and deleting domain"""
        # Create domain
        d_name = data_utils.rand_name('domain')
        d_desc = data_utils.rand_name('domain-desc')
        domain = self.domains_client.create_domain(
            name=d_name, description=d_desc)['domain']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.delete_domain, domain['id'])
        self.assertIn('description', domain)
        self.assertIn('name', domain)
        self.assertIn('enabled', domain)
        self.assertIn('links', domain)
        self.assertIsNotNone(domain['id'])
        self.assertEqual(d_name, domain['name'])
        self.assertEqual(d_desc, domain['description'])
        self.assertEqual(True, domain['enabled'])
        # Update domain
        new_desc = data_utils.rand_name('new-desc')
        new_name = data_utils.rand_name('new-name')
        updated_domain = self.domains_client.update_domain(
            domain['id'], name=new_name, description=new_desc,
            enabled=False)['domain']
        self.assertIn('id', updated_domain)
        self.assertIn('description', updated_domain)
        self.assertIn('name', updated_domain)
        self.assertIn('enabled', updated_domain)
        self.assertIn('links', updated_domain)
        self.assertIsNotNone(updated_domain['id'])
        self.assertEqual(new_name, updated_domain['name'])
        self.assertEqual(new_desc, updated_domain['description'])
        self.assertEqual(False, updated_domain['enabled'])
        # Show domain
        fetched_domain = self.domains_client.show_domain(
            domain['id'])['domain']
        self.assertEqual(new_name, fetched_domain['name'])
        self.assertEqual(new_desc, fetched_domain['description'])
        self.assertEqual(False, fetched_domain['enabled'])
        # Delete domain
        self.domains_client.delete_domain(domain['id'])
        body = self.domains_client.list_domains()['domains']
        domains_list = [d['id'] for d in body]
        self.assertNotIn(domain['id'], domains_list)

    @decorators.idempotent_id('d8d318b7-d1b3-4c37-94c5-3c5ba0b121ea')
    def test_domain_delete_cascades_content(self):
        """Test deleting domain will delete its associated contents"""
        # Create a domain with a user and a group in it
        domain = self.setup_test_domain()
        user = self.create_test_user(domain_id=domain['id'])
        group = self.setup_test_group(domain_id=domain['id'])
        # Delete the domain
        self.delete_domain(domain['id'])
        # Check the domain, its users and groups are gone
        self.assertRaises(exceptions.NotFound,
                          self.domains_client.show_domain, domain['id'])
        self.assertRaises(exceptions.NotFound,
                          self.users_client.show_user, user['id'])
        self.assertRaises(exceptions.NotFound,
                          self.groups_client.show_group, group['id'])

    @decorators.idempotent_id('036df86e-bb5d-42c0-a7c2-66b9db3a6046')
    def test_create_domain_with_disabled_status(self):
        """Test creating domain with disabled status"""
        # Create domain with enabled status as false
        d_name = data_utils.rand_name('domain')
        d_desc = data_utils.rand_name('domain-desc')
        domain = self.domains_client.create_domain(
            name=d_name, description=d_desc, enabled=False)['domain']
        self.addCleanup(self.domains_client.delete_domain, domain['id'])
        self.assertEqual(d_name, domain['name'])
        self.assertFalse(domain['enabled'])
        self.assertEqual(d_desc, domain['description'])

    @decorators.idempotent_id('2abf8764-309a-4fa9-bc58-201b799817ad')
    def test_create_domain_without_description(self):
        """Test creating domain without description"""
        # Create domain only with name
        d_name = data_utils.rand_name('domain')
        domain = self.domains_client.create_domain(name=d_name)['domain']
        self.addCleanup(self.delete_domain, domain['id'])
        expected_data = {'name': d_name, 'enabled': True}
        self.assertEqual('', domain['description'])
        self.assertDictContainsSubset(expected_data, domain)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import itertools
import random
import string
import uuid

from oslo_utils import uuidutils
import six.moves


def rand_uuid():
    """Generate a random UUID string

    :return: a random UUID (e.g. '1dc12c7d-60eb-4b61-a7a2-17cf210155b6')
    :rtype: string
    """
    return uuidutils.generate_uuid()


def rand_uuid_hex():
    """Generate a random UUID hex string

    :return: a random UUID (e.g. '0b98cf96d90447bda4b46f31aeb1508c')
    :rtype: string
    """
    return uuid.uuid4().hex


def rand_name(name='', prefix='tempest'):
    """Generate a random name that includes a random number

    :param str name: The name that you want to include
    :param str prefix: The prefix that you want to include
    :return: a random name. The format is
             '<prefix>-<name>-<random number>'.
             (e.g. 'prefixfoo-namebar-154876201')
    :rtype: string
    """
    rand_name = str(random.randint(1, 0x7fffffff))
    if name:
        rand_name = name + '-' + rand_name
    if prefix:
        rand_name = prefix + '-' + rand_name
    return rand_name


def rand_password(length=15):
    """Generate a random password

    :param int length: The length of password that you expect to set
                       (If it's smaller than 3, it's same as 3.)
    :return: a random password. The format is
        ``'<random upper letter>-<random number>-<random special character>
        -<random ascii letters or digit characters or special symbols>'``
        (e.g. ``G2*ac8&lKFFgh%2``)
    :rtype: string
    """
    upper = random.choice(string.ascii_uppercase)
    ascii_char = string.ascii_letters
    digits = string.digits
    digit = random.choice(string.digits)
    puncs = '~!@#%^&*_=+'
    punc = random.choice(puncs)
    seed = ascii_char + digits + puncs
    pre = upper + digit + punc
    password = pre + ''.join(random.choice(seed) for x in range(length - 3))
    return password


def rand_url():
    """Generate a random url that includes a random number

    :return: a random url. The format is 'https://url-<random number>.com'.
             (e.g. 'https://url-154876201.com')
    :rtype: string
    """
    randbits = str(random.randint(1, 0x7fffffff))
    return 'https://url-' + randbits + '.com'


def rand_int_id(start=0, end=0x7fffffff):
    """Generate a random integer value

    :param int start: The value that you expect to start here
    :param int end: The value that you expect to end here
    :return: a random integer value
    :rtype: int
    """
    return random.randint(start, end)


def rand_mac_address():
    """Generate an Ethernet MAC address

    :return: an random Ethernet MAC address
    :rtype: string
    """
    # NOTE(vish): We would prefer to use 0xfe here to ensure that linux
    #             bridge mac addresses don't change, but it appears to
    #             conflict with libvirt, so we use the next highest octet
    #             that has the unicast and locally administered bits set
    #             properly: 0xfa.
    #             Discussion: https://bugs.launchpad.net/nova/+bug/921838
    mac = [0xfa, 0x16, 0x3e,
           random.randint(0x00, 0xff),
           random.randint(0x00, 0xff),
           random.randint(0x00, 0xff)]
    return ':'.join(["%02x" % x for x in mac])


def rand_infiniband_guid_address():
    """Generate an Infiniband GUID address

    :return: an random Infiniband GUID address
    :rtype: string
    """
    guid = []
    for _ in range(8):
        guid.append("%02x" % random.randint(0x00, 0xff))
    return ':'.join(guid)


def parse_image_id(image_ref):
    """Return the image id from a given image ref

    This function just returns the last word of the given image ref string
    splitting with '/'.
    :param str image_ref: a string that includes the image id
    :return: the image id string
    :rtype: string
    """
    return image_ref.rsplit('/')[-1]


def arbitrary_string(size=4, base_text=None):
    """Return size characters from base_text

    This generates a string with an arbitrary number of characters, generated
    by looping the base_text string. If the size is smaller than the size of
    base_text, returning string is shrunk to the size.
    :param int size: a returning characters size
    :param str base_text: a string you want to repeat
    :return: size string
    :rtype: string
    """
    if not base_text:
        base_text = 'test'
    return ''.join(itertools.islice(itertools.cycle(base_text), size))


def random_bytes(size=1024):
    """Return size randomly selected bytes as a string

    :param int size: a returning bytes size
    :return: size randomly bytes
    :rtype: string
    """
    if size > 1 << 20:
        raise RuntimeError('Size should be less than 1MiB')
    return b''.join([six.int2byte(random.randint(0, 255))
                     for i in range(size)])


# Courtesy of http://stackoverflow.com/a/312464
def chunkify(sequence, chunksize):
    """Yield successive chunks from `sequence`."""
    for i in range(0, len(sequence), chunksize):
        yield sequence[i:i + chunksize]



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class ListImagesTestJSON(base.BaseV2ComputeTest):
    """Test listing server images with compute microversion less than 2.36"""

    max_microversion = '2.35'

    @classmethod
    def skip_checks(cls):
        super(ListImagesTestJSON, cls).skip_checks()
        if not CONF.service_available.glance:
            skip_msg = ("%s skipped as glance is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(ListImagesTestJSON, cls).setup_clients()
        cls.client = cls.compute_images_client

    @decorators.idempotent_id('490d0898-e12a-463f-aef0-c50156b9f789')
    def test_get_image(self):
        """Test getting the correct details for a single server image"""
        image = self.client.show_image(self.image_ref)['image']
        self.assertEqual(self.image_ref, image['id'])

    @decorators.idempotent_id('fd51b7f4-d4a3-4331-9885-866658112a6f')
    def test_list_images(self):
        """Test listing server images

        The list of all images should contain the image
        """
        images = self.client.list_images()['images']
        found = [i for i in images if i['id'] == self.image_ref]
        self.assertNotEmpty(found)

    @decorators.idempotent_id('9f94cb6b-7f10-48c5-b911-a0b84d7d4cd6')
    def test_list_images_with_detail(self):
        """Test listing server images with detail

        Detailed list of all images should contain the expected images
        """
        images = self.client.list_images(detail=True)['images']
        found = [i for i in images if i['id'] == self.image_ref]
        self.assertNotEmpty(found)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.image import base
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators


class MetadataNamespaceTagsTest(base.BaseV2ImageAdminTest):
    """Test the Metadata definition namespace tags basic functionality"""

    tags = [
        {
            "name": "sample-tag1"
        },
        {
            "name": "sample-tag2"
        },
        {
            "name": "sample-tag3"
        }
    ]
    tag_list = ["sample-tag1", "sample-tag2", "sample-tag3"]

    def _create_namespace_tags(self, namespace):
        # Create a namespace
        namespace_tags = self.namespace_tags_client.create_namespace_tags(
            namespace['namespace'], tags=self.tags)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.namespace_tags_client.delete_namespace_tags,
                        namespace['namespace'])
        return namespace_tags

    @decorators.idempotent_id('a2a3765e-3a6d-4f6d-a3a7-3cc3476aa876')
    def test_create_list_delete_namespace_tags(self):
        """Test creating/listing/deleting image metadata namespace tags"""
        # Create a namespace
        namespace = self.create_namespace()
        self._create_namespace_tags(namespace)
        # List namespace tags
        body = self.namespace_tags_client.list_namespace_tags(
            namespace['namespace'])
        self.assertEqual(3, len(body['tags']))
        self.assertIn(body['tags'][0]['name'], self.tag_list)
        self.assertIn(body['tags'][1]['name'], self.tag_list)
        self.assertIn(body['tags'][2]['name'], self.tag_list)
        # Delete all tag definitions
        self.namespace_tags_client.delete_namespace_tags(
            namespace['namespace'])
        body = self.namespace_tags_client.list_namespace_tags(
            namespace['namespace'])
        self.assertEmpty(body['tags'])

    @decorators.idempotent_id('a2a3765e-1a2c-3f6d-a3a7-3cc3466ab875')
    def test_create_update_delete_tag(self):
        """Test creating/updating/deleting image metadata namespace tag"""
        # Create a namespace
        namespace = self.create_namespace()
        self._create_namespace_tags(namespace)
        # Create a tag
        tag_name = data_utils.rand_name('tag_name')
        self.namespace_tags_client.create_namespace_tag(
            namespace=namespace['namespace'], tag_name=tag_name)

        body = self.namespace_tags_client.show_namespace_tag(
            namespace['namespace'], tag_name)
        self.assertEqual(tag_name, body['name'])
        # Update tag definition
        update_tag_definition = data_utils.rand_name('update-tag')
        body = self.namespace_tags_client.update_namespace_tag(
            namespace['namespace'], tag_name=tag_name,
            name=update_tag_definition)
        self.assertEqual(update_tag_definition, body['name'])
        # Delete tag definition
        self.namespace_tags_client.delete_namespace_tag(
            namespace['namespace'], update_tag_definition)
        # List namespace tags and validate deletion
        namespace_tags = [
            namespace_tag['name'] for namespace_tag in
            self.namespace_tags_client.list_namespace_tags(
                namespace['namespace'])['tags']]
        self.assertNotIn(update_tag_definition, namespace_tags)



# Copyright 2017 AT&T Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class TokensAdminTestNegative(base.BaseIdentityV2AdminTest):
    """Negative tests of keystone tokens via v2 API"""

    credentials = ['primary', 'admin', 'alt']

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a0a0a600-4292-4364-99c5-922c834fdf05')
    def test_check_token_existence_negative(self):
        """Test checking other tenant's token existence via v2 API

        Checking other tenant's token existence via v2 API should fail.
        """
        creds = self.os_primary.credentials
        creds_alt = self.os_alt.credentials
        username = creds.username
        password = creds.password
        tenant_name = creds.tenant_name
        alt_tenant_name = creds_alt.tenant_name
        body = self.token_client.auth(username, password, tenant_name)
        self.assertRaises(lib_exc.Unauthorized,
                          self.client.check_token_existence,
                          body['token']['id'],
                          belongsTo=alt_tenant_name)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

create_server = {
    'status_code': [202],
    'response_body': {
        'type': 'object',
        'properties': {
            'server': {
                'type': 'object',
                'properties': {
                    'id': {'type': 'string'},
                    'security_groups': {'type': 'array'},
                    'links': parameter_types.links,
                    'OS-DCF:diskConfig': {'type': 'string'}
                },
                'additionalProperties': False,
                # NOTE: OS-DCF:diskConfig & security_groups are API extension,
                # and some environments return a response without these
                # attributes.So they are not 'required'.
                'required': ['id', 'links']
            }
        },
        'additionalProperties': False,
        'required': ['server']
    }
}

create_server_with_admin_pass = copy.deepcopy(create_server)
create_server_with_admin_pass['response_body']['properties']['server'][
    'properties'].update({'adminPass': {'type': 'string'}})
create_server_with_admin_pass['response_body']['properties']['server'][
    'required'].append('adminPass')

list_servers = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'servers': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'id': {'type': 'string'},
                        'links': parameter_types.links,
                        'name': {'type': 'string'}
                    },
                    'additionalProperties': False,
                    'required': ['id', 'links', 'name']
                }
            },
            'servers_links': parameter_types.links
        },
        'additionalProperties': False,
        # NOTE(gmann): servers_links attribute is not necessary to be
        # present always So it is not 'required'.
        'required': ['servers']
    }
}

delete_server = {
    'status_code': [204],
}

common_show_server = {
    'type': 'object',
    'properties': {
        'id': {'type': 'string'},
        'name': {'type': 'string'},
        'status': {'type': 'string'},
        'image': {'oneOf': [
            {'type': 'object',
                'properties': {
                    'id': {'type': 'string'},
                    'links': parameter_types.links
                },
                'additionalProperties': False,
                'required': ['id', 'links']},
            {'type': ['string', 'null']}
        ]},
        'flavor': {
            'type': 'object',
            'properties': {
                'id': {'type': 'string'},
                'links': parameter_types.links
            },
            # NOTE(gmann): This will be empty object if there is no
            # flavor info present in DB. This can happen when flavor info is
            # deleted after server creation.
            'additionalProperties': False
        },
        'fault': {
            'type': 'object',
            'properties': {
                'code': {'type': 'integer'},
                'created': {'type': 'string'},
                'message': {'type': 'string'},
                'details': {'type': 'string'},
            },
            'additionalProperties': False,
            # NOTE(gmann): 'details' is not necessary to be present
            #  in the 'fault'. So it is not defined as 'required'.
            'required': ['code', 'created', 'message']
        },
        'user_id': {'type': 'string'},
        'tenant_id': {'type': 'string'},
        'created': parameter_types.date_time,
        'updated': parameter_types.date_time,
        'progress': {'type': 'integer'},
        'metadata': {'type': 'object'},
        'links': parameter_types.links,
        'addresses': parameter_types.addresses,
        'hostId': {'type': 'string'},
        'OS-DCF:diskConfig': {'type': 'string'},
        'accessIPv4': parameter_types.access_ip_v4,
        'accessIPv6': parameter_types.access_ip_v6
    },
    'additionalProperties': False,
    # NOTE(GMann): 'progress' attribute is present in the response
    # only when server's status is one of the progress statuses
    # ("ACTIVE","BUILD", "REBUILD", "RESIZE","VERIFY_RESIZE")
    # 'fault' attribute is present in the response
    # only when server's status is one of the  "ERROR", "DELETED".
    # OS-DCF:diskConfig and accessIPv4/v6 are API
    # extensions, and some environments return a response
    # without these attributes.So these are not defined as 'required'.
    'required': ['id', 'name', 'status', 'image', 'flavor',
                 'user_id', 'tenant_id', 'created', 'updated',
                 'metadata', 'links', 'addresses', 'hostId']
}

update_server = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'server': common_show_server
        },
        'additionalProperties': False,
        'required': ['server']
    }
}

server_detail = copy.deepcopy(common_show_server)
server_detail['properties'].update({
    'key_name': {'type': ['string', 'null']},
    'security_groups': {'type': 'array'},

    # NOTE: Non-admin users also can see "OS-SRV-USG" and "OS-EXT-AZ"
    # attributes.
    'OS-SRV-USG:launched_at': {'type': ['string', 'null']},
    'OS-SRV-USG:terminated_at': {'type': ['string', 'null']},
    'OS-EXT-AZ:availability_zone': {'type': 'string'},

    # NOTE: Admin users only can see "OS-EXT-STS" and "OS-EXT-SRV-ATTR"
    # attributes.
    'OS-EXT-STS:task_state': {'type': ['string', 'null']},
    'OS-EXT-STS:vm_state': {'type': 'string'},
    'OS-EXT-STS:power_state': parameter_types.power_state,
    'OS-EXT-SRV-ATTR:host': {'type': ['string', 'null']},
    'OS-EXT-SRV-ATTR:instance_name': {'type': 'string'},
    'OS-EXT-SRV-ATTR:hypervisor_hostname': {'type': ['string', 'null']},
    'os-extended-volumes:volumes_attached': {
        'type': 'array',
        'items': {
            'type': 'object',
            'properties': {
                'id': {'type': 'string'}
            },
            'additionalProperties': False,
        },
    },
    'config_drive': {'type': 'string'}
})
server_detail['properties']['addresses']['patternProperties'][
    '^[a-zA-Z0-9-_.]+$']['items']['properties'].update({
        'OS-EXT-IPS:type': {'type': 'string'},
        'OS-EXT-IPS-MAC:mac_addr': parameter_types.mac_address})
# NOTE(gmann): Update OS-EXT-IPS:type and OS-EXT-IPS-MAC:mac_addr
# attributes in server address. Those are API extension,
# and some environments return a response without
# these attributes. So they are not 'required'.

get_server = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'server': server_detail
        },
        'additionalProperties': False,
        'required': ['server']
    }
}

list_servers_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'servers': {
                'type': 'array',
                'items': server_detail
            },
            'servers_links': parameter_types.links
        },
        'additionalProperties': False,
        # NOTE(gmann): servers_links attribute is not necessary to be
        # present always So it is not 'required'.
        'required': ['servers']
    }
}

rebuild_server = copy.deepcopy(update_server)
rebuild_server['status_code'] = [202]

rebuild_server_with_admin_pass = copy.deepcopy(rebuild_server)
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'properties'].update({'adminPass': {'type': 'string'}})
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'required'].append('adminPass')

rescue_server = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'additionalProperties': False,
    }
}

rescue_server_with_admin_pass = copy.deepcopy(rescue_server)
rescue_server_with_admin_pass['response_body'].update(
    {'properties': {'adminPass': {'type': 'string'}}})
rescue_server_with_admin_pass['response_body'].update(
    {'required': ['adminPass']})


list_virtual_interfaces = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'virtual_interfaces': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'id': {'type': 'string'},
                        'mac_address': parameter_types.mac_address,
                        'OS-EXT-VIF-NET:net_id': {'type': 'string'}
                    },
                    'additionalProperties': False,
                    # 'OS-EXT-VIF-NET:net_id' is API extension So it is
                    # not defined as 'required'
                    'required': ['id', 'mac_address']
                }
            }
        },
        'additionalProperties': False,
        'required': ['virtual_interfaces']
    }
}

common_attach_volume_info = {
    'type': 'object',
    'properties': {
        'id': {'type': 'string'},
        'device': {'type': ['string', 'null']},
        'volumeId': {'type': 'string'},
        'serverId': {'type': ['integer', 'string']}
    },
    'additionalProperties': False,
    # 'device' is optional in response.
    'required': ['id', 'volumeId', 'serverId']
}

attach_volume = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'volumeAttachment': common_attach_volume_info
        },
        'additionalProperties': False,
        'required': ['volumeAttachment']
    }
}

detach_volume = {
    'status_code': [202]
}

show_volume_attachment = copy.deepcopy(attach_volume)
show_volume_attachment['response_body']['properties'][
    'volumeAttachment']['properties'].update({'serverId': {'type': 'string'}})

list_volume_attachments = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'volumeAttachments': {
                'type': 'array',
                'items': common_attach_volume_info
            }
        },
        'additionalProperties': False,
        'required': ['volumeAttachments']
    }
}
list_volume_attachments['response_body']['properties'][
    'volumeAttachments']['items']['properties'].update(
    {'serverId': {'type': 'string'}})

list_addresses_by_network = {
    'status_code': [200],
    'response_body': parameter_types.addresses
}

list_addresses = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'addresses': parameter_types.addresses
        },
        'additionalProperties': False,
        'required': ['addresses']
    }
}

instance_actions = {
    'type': 'object',
    'properties': {
        'action': {'type': 'string'},
        'request_id': {'type': 'string'},
        'user_id': {'type': 'string'},
        'project_id': {'type': 'string'},
        'start_time': parameter_types.date_time,
        'message': {'type': ['string', 'null']},
        'instance_uuid': {'type': 'string'}
    },
    'additionalProperties': False,
    'required': ['action', 'request_id', 'user_id', 'project_id',
                 'start_time', 'message', 'instance_uuid']
}

instance_action_events = {
    'type': 'array',
    'items': {
        'type': 'object',
        'properties': {
            'event': {'type': 'string'},
            'start_time': parameter_types.date_time,
            # The finish_time, result and optionally traceback are all
            # possibly None (null) until the event is actually finished.
            # The traceback would only be set if there was an error, but
            # when the event is complete both finish_time and result will
            # be set.
            'finish_time': parameter_types.date_time_or_null,
            'result': {'type': ['string', 'null']},
            'traceback': {'type': ['string', 'null']}
        },
        'additionalProperties': False,
        # NOTE(zhufl): events.traceback can only be seen by admin users
        # with default policy.json, so it shouldn't be a required field.
        'required': ['event', 'start_time', 'finish_time', 'result']
    }
}

list_instance_actions = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'instanceActions': {
                'type': 'array',
                'items': instance_actions
            }
        },
        'additionalProperties': False,
        'required': ['instanceActions']
    }
}

instance_actions_with_events = copy.deepcopy(instance_actions)
instance_actions_with_events['properties'].update({
    'events': instance_action_events})
# 'events' does not come in response body always so it is not
# defined as 'required'

show_instance_action = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'instanceAction': instance_actions_with_events
        },
        'additionalProperties': False,
        'required': ['instanceAction']
    }
}

show_password = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'password': {'type': 'string'}
        },
        'additionalProperties': False,
        'required': ['password']
    }
}

get_vnc_console = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'console': {
                'type': 'object',
                'properties': {
                    'type': {'type': 'string'},
                    'url': {
                        'type': 'string',
                        'format': 'uri'
                    }
                },
                'additionalProperties': False,
                'required': ['type', 'url']
            }
        },
        'additionalProperties': False,
        'required': ['console']
    }
}

get_console_output = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'output': {'type': 'string'}
        },
        'additionalProperties': False,
        'required': ['output']
    }
}

set_server_metadata = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'metadata': {
                'type': 'object',
                'patternProperties': {
                    '^.+$': {'type': 'string'}
                }
            }
        },
        'additionalProperties': False,
        'required': ['metadata']
    }
}

list_server_metadata = copy.deepcopy(set_server_metadata)

update_server_metadata = copy.deepcopy(set_server_metadata)

delete_server_metadata_item = {
    'status_code': [204]
}

set_show_server_metadata_item = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'meta': {
                'type': 'object',
                'patternProperties': {
                    '^.+$': {'type': 'string'}
                }
            }
        },
        'additionalProperties': False,
        'required': ['meta']
    }
}

server_actions_common_schema = {
    'status_code': [202]
}

server_actions_delete_password = {
    'status_code': [204]
}

server_actions_confirm_resize = copy.deepcopy(
    server_actions_delete_password)

update_attached_volume = {
    'status_code': [202]
}

evacuate_server = {
    'status_code': [200]
}

evacuate_server_with_admin_pass = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'adminPass': {'type': 'string'}
        },
        'additionalProperties': False,
        'required': ['adminPass']
    }
}

show_server_diagnostics = {
    'status_code': [200],
    'response_body': {
        'type': 'object'
    }
}



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io

from tempest.api.compute import base
from tempest.common import image as common_image
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF


class ImagesMetadataTestJSON(base.BaseV2ComputeTest):
    """Test image metadata with compute microversion less than 2.39"""

    max_microversion = '2.38'

    @classmethod
    def skip_checks(cls):
        super(ImagesMetadataTestJSON, cls).skip_checks()
        if not CONF.service_available.glance:
            skip_msg = ("%s skipped as glance is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(ImagesMetadataTestJSON, cls).setup_clients()
        # Check if glance v1 is available to determine which client to use. We
        # prefer glance v1 for the compute API tests since the compute image
        # API proxy was written for glance v1.
        if CONF.image_feature_enabled.api_v1:
            cls.glance_client = cls.os_primary.image_client
        elif CONF.image_feature_enabled.api_v2:
            cls.glance_client = cls.os_primary.image_client_v2
        else:
            raise exceptions.InvalidConfiguration(
                'Either api_v1 or api_v2 must be True in '
                '[image-feature-enabled].')
        cls.client = cls.compute_images_client

    @classmethod
    def resource_setup(cls):
        super(ImagesMetadataTestJSON, cls).resource_setup()
        cls.image_id = None

        params = {
            'name': data_utils.rand_name('image'),
            'container_format': 'bare',
            'disk_format': 'raw'
        }
        if CONF.image_feature_enabled.api_v1:
            params.update({'is_public': False})
            params = {'headers': common_image.image_meta_to_headers(**params)}
        else:
            params.update({'visibility': 'private'})

        body = cls.glance_client.create_image(**params)
        body = body['image'] if 'image' in body else body
        cls.image_id = body['id']
        cls.addClassResourceCleanup(test_utils.call_and_ignore_notfound_exc,
                                    cls.glance_client.delete_image,
                                    cls.image_id)
        image_file = io.BytesIO((b'*' * 1024))
        if CONF.image_feature_enabled.api_v1:
            cls.glance_client.update_image(cls.image_id, data=image_file)
        else:
            cls.glance_client.store_image_file(cls.image_id, data=image_file)
        waiters.wait_for_image_status(cls.client, cls.image_id, 'ACTIVE')

    def setUp(self):
        super(ImagesMetadataTestJSON, self).setUp()
        meta = {'os_version': 'value1', 'os_distro': 'value2'}
        self.client.set_image_metadata(self.image_id, meta)

    @decorators.idempotent_id('37ec6edd-cf30-4c53-bd45-ae74db6b0531')
    def test_list_image_metadata(self):
        """Test listing image metadata

        All metadata key/value pairs for an image should be returned.
        """
        resp_metadata = self.client.list_image_metadata(self.image_id)
        expected = {'metadata': {
            'os_version': 'value1', 'os_distro': 'value2'}}
        self.assertEqual(expected, resp_metadata)

    @decorators.idempotent_id('ece7befc-d3ce-42a4-b4be-c3067a418c29')
    def test_set_image_metadata(self):
        """Test setting image metadata

        The metadata for the image should match the new values.
        """
        req_metadata = {'os_version': 'value2', 'architecture': 'value3'}
        self.client.set_image_metadata(self.image_id,
                                       req_metadata)

        resp_metadata = (self.client.list_image_metadata(self.image_id)
                         ['metadata'])
        self.assertEqual(req_metadata, resp_metadata)

    @decorators.idempotent_id('7b491c11-a9d5-40fe-a696-7f7e03d3fea2')
    def test_update_image_metadata(self):
        """Test updating image medata

        The metadata for the image should match the updated values.
        """
        req_metadata = {'os_version': 'alt1', 'architecture': 'value3'}
        self.client.update_image_metadata(self.image_id,
                                          req_metadata)

        resp_metadata = self.client.list_image_metadata(self.image_id)
        expected = {'metadata': {
            'os_version': 'alt1',
            'os_distro': 'value2',
            'architecture': 'value3'}}
        self.assertEqual(expected, resp_metadata)

    @decorators.idempotent_id('4f5db52f-6685-4c75-b848-f4bb363f9aa6')
    def test_get_image_metadata_item(self):
        """Test getting image metadata item

        The value for a specific metadata key should be returned.
        """
        meta = self.client.show_image_metadata_item(self.image_id,
                                                    'os_distro')['meta']
        self.assertEqual('value2', meta['os_distro'])

    @decorators.idempotent_id('f2de776a-4778-4d90-a5da-aae63aee64ae')
    def test_set_image_metadata_item(self):
        """Test setting image metadata item

        The value provided for the given meta item should be set for
        the image.
        """
        meta = {'os_version': 'alt'}
        self.client.set_image_metadata_item(self.image_id,
                                            'os_version', meta)
        resp_metadata = self.client.list_image_metadata(self.image_id)
        expected = {'metadata': {'os_version': 'alt', 'os_distro': 'value2'}}
        self.assertEqual(expected, resp_metadata)

    @decorators.idempotent_id('a013796c-ba37-4bb5-8602-d944511def14')
    def test_delete_image_metadata_item(self):
        """Test deleting image metadata item

        The metadata value/key pair should be deleted from the image.
        """
        self.client.delete_image_metadata_item(self.image_id,
                                               'os_version')
        resp_metadata = self.client.list_image_metadata(self.image_id)
        expected = {'metadata': {'os_distro': 'value2'}}
        self.assertEqual(expected, resp_metadata)



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import inspect
import re
import time

from oslo_log import log as logging

from tempest.lib import exceptions

LOG = logging.getLogger(__name__)


def find_test_caller():
    """Find the caller class and test name.

    Because we know that the interesting things that call us are
    test_* methods, and various kinds of setUp / tearDown, we
    can look through the call stack to find appropriate methods,
    and the class we were in when those were called.
    """
    caller_name = None
    names = []
    frame = inspect.currentframe()
    is_cleanup = False
    # Start climbing the ladder until we hit a good method
    while True:
        try:
            frame = frame.f_back
            name = frame.f_code.co_name
            names.append(name)
            if re.search("^(test_|setUp|tearDown)", name):
                cname = ""
                if 'self' in frame.f_locals:
                    cname = frame.f_locals['self'].__class__.__name__
                if 'cls' in frame.f_locals:
                    cname = frame.f_locals['cls'].__name__
                caller_name = cname + ":" + name
                break
            elif re.search("^_run_cleanup", name):
                is_cleanup = True
            elif name == 'main':
                caller_name = 'main'
                break
            else:
                cname = ""
                if 'self' in frame.f_locals:
                    cname = frame.f_locals['self'].__class__.__name__
                if 'cls' in frame.f_locals:
                    cname = frame.f_locals['cls'].__name__

                # the fact that we are running cleanups is indicated pretty
                # deep in the stack, so if we see that we want to just
                # start looking for a real class name, and declare victory
                # once we do.
                if is_cleanup and cname:
                    if not re.search("^RunTest", cname):
                        caller_name = cname + ":_run_cleanups"
                        break
        except Exception:
            break
    # prevents frame leaks
    del frame
    if caller_name is None:
        LOG.debug("Sane call name not found in %s", names)
    return caller_name


def call_and_ignore_notfound_exc(func, *args, **kwargs):
    """Call the given function and pass if a `NotFound` exception is raised."""
    attempt = 0
    while True:
        attempt += 1
        try:
            return func(*args, **kwargs)
        except exceptions.NotFound:
            return
        except exceptions.ServerFault:
            # NOTE(danms): Tolerate three ServerFault exceptions while trying
            # to do this thing, and after that, assume it's legit.
            if attempt >= 3:
                raise
            LOG.warning('Got ServerFault while running %s, retrying...', func)


def call_until_true(func, duration, sleep_for, *args, **kwargs):
    """Call the given function until it returns True (and return True)

    or until the specified duration (in seconds) elapses (and return False).

    :param func: A callable that returns True on success.
    :param duration: The number of seconds for which to attempt a
        successful call of the function.
    :param sleep_for: The number of seconds to sleep after an unsuccessful
                      invocation of the function.
    :param args: args that are passed to func.
    :param kwargs: kwargs that are passed to func.
    """
    now = time.time()
    begin_time = now
    timeout = now + duration
    func_name = getattr(func, '__name__', getattr(func.__class__, '__name__'))
    while now < timeout:
        if func(*args, **kwargs):
            LOG.debug("Call %s returns true in %f seconds",
                      func_name, time.time() - begin_time)
            return True
        time.sleep(sleep_for)
        now = time.time()
    LOG.debug("Call %s returns false in %f seconds", func_name, duration)
    return False



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import compute
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class DeleteServersTestJSON(base.BaseV2ComputeTest):
    """Test deleting servers in various states"""
    create_default_network = True

    # NOTE: Server creations of each test class should be under 10
    # for preventing "Quota exceeded for instances"

    @classmethod
    def setup_clients(cls):
        super(DeleteServersTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @decorators.idempotent_id('9e6e0c87-3352-42f7-9faf-5d6210dbd159')
    def test_delete_server_while_in_building_state(self):
        """Test deleting a server while it's VM state is Building"""
        server = self.create_test_server(wait_until='BUILD')
        self.client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.client, server['id'])

    @decorators.idempotent_id('925fdfb4-5b13-47ea-ac8a-c36ae6fddb05')
    def test_delete_active_server(self):
        """Test deleting a server while it's VM state is Active"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.client, server['id'])

    @decorators.idempotent_id('546d368c-bb6c-4645-979a-83ed16f3a6be')
    def test_delete_server_while_in_shutoff_state(self):
        """Test deleting a server while it's VM state is Shutoff"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.client.stop_server(server['id'])
        waiters.wait_for_server_status(self.client, server['id'], 'SHUTOFF')
        self.client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.client, server['id'])

    @decorators.idempotent_id('943bd6e8-4d7a-4904-be83-7a6cc2d4213b')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    def test_delete_server_while_in_pause_state(self):
        """Test deleting a server while it's VM state is Pause"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.client.pause_server(server['id'])
        waiters.wait_for_server_status(self.client, server['id'], 'PAUSED')
        self.client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.client, server['id'])

    @decorators.idempotent_id('1f82ebd3-8253-4f4e-b93f-de9b7df56d8b')
    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          'Suspend is not available.')
    def test_delete_server_while_in_suspended_state(self):
        """Test deleting a server while it's VM state is Suspended"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.client.suspend_server(server['id'])
        waiters.wait_for_server_status(self.client, server['id'], 'SUSPENDED')
        self.client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.client, server['id'])

    @decorators.idempotent_id('bb0cb402-09dd-4947-b6e5-5e7e1cfa61ad')
    @testtools.skipUnless(CONF.compute_feature_enabled.shelve,
                          'Shelve is not available.')
    def test_delete_server_while_in_shelved_state(self):
        """Test deleting a server while it's VM state is Shelved"""
        server = self.create_test_server(wait_until='ACTIVE')
        compute.shelve_server(self.client, server['id'])

        self.client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.client, server['id'])

    @decorators.idempotent_id('ab0c38b4-cdd8-49d3-9b92-0cb898723c01')
    @testtools.skipIf(not CONF.compute_feature_enabled.resize,
                      'Resize not available.')
    def test_delete_server_while_in_verify_resize_state(self):
        """Test deleting a server while it's VM state is VERIFY_RESIZE"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.client.resize_server(server['id'], self.flavor_ref_alt)
        waiters.wait_for_server_status(self.client, server['id'],
                                       'VERIFY_RESIZE')
        self.client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.client, server['id'])

    @decorators.idempotent_id('d0f3f0d6-d9b6-4a32-8da4-23015dcab23c')
    @utils.services('volume')
    def test_delete_server_while_in_attached_volume(self):
        """Test deleting a server while a volume is attached to it"""
        server = self.create_test_server(wait_until='ACTIVE')

        volume = self.create_volume()
        self.attach_volume(server, volume)

        self.client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.client, server['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.network import base
from tempest.common import utils
from tempest.lib import decorators


class ServiceProvidersTest(base.BaseNetworkTest):
    """Test network service providers"""

    @decorators.idempotent_id('2cbbeea9-f010-40f6-8df5-4eaa0c918ea6')
    @testtools.skipUnless(
        utils.is_extension_enabled('service-type', 'network'),
        'service-type extension not enabled.')
    def test_service_providers_list(self):
        """Test listing network service providers"""
        body = self.service_providers_client.list_service_providers()
        self.assertIn('service_providers', body)
        self.assertIsInstance(body['service_providers'], list)



# Copyright 2017 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.image import base
from tempest.lib import decorators


class VersionsTest(base.BaseV2ImageTest):
    """Test image versions"""

    @decorators.idempotent_id('659ea30a-a17c-4317-832c-0f68ed23c31d')
    @decorators.attr(type='smoke')
    def test_list_versions(self):
        """Test listing image versions"""
        versions = self.versions_client.list_versions()['versions']
        expected_resources = ('id', 'links', 'status')

        for version in versions:
            for res in expected_resources:
                self.assertIn(res, version)



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class IdentityTenantsTest(base.BaseIdentityV2Test):
    """Test listing tenants in identity v2 API"""

    credentials = ['primary', 'alt']

    @decorators.idempotent_id('ecae2459-243d-4ba1-ad02-65f15dc82b78')
    def test_list_tenants_returns_only_authorized_tenants(self):
        """Test listing tenants only returns authorized tenants via v2 API"""
        alt_tenant_name = self.os_alt.credentials.tenant_name
        resp = self.non_admin_tenants_client.list_tenants()

        # check that user can see only that tenants that he presents in so user
        # can successfully authenticate using his credentials and tenant name
        # from received tenants list
        for tenant in resp['tenants']:
            body = self.non_admin_token_client.auth(
                self.os_primary.credentials.username,
                self.os_primary.credentials.password,
                tenant['name'])
            self.assertNotEmpty(body['token']['id'])
            self.assertEqual(body['token']['tenant']['id'], tenant['id'])
            self.assertEqual(body['token']['tenant']['name'], tenant['name'])
            self.assertEqual(
                body['user']['id'], self.os_primary.credentials.user_id)

        # check that user cannot log in to alt user's tenant
        self.assertRaises(
            lib_exc.Unauthorized,
            self.non_admin_token_client.auth,
            self.os_primary.credentials.username,
            self.os_primary.credentials.password,
            alt_tenant_name)



# Copyright 2016 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_config import cfg

from tempest import clients
from tempest.common import credentials_factory as credentials
from tempest import config
from tempest.lib.common import fixed_network
from tempest import test
from tempest.tests import base
from tempest.tests import fake_config


class TestBaseTestCase(base.TestCase):
    def setUp(self):
        super(TestBaseTestCase, self).setUp()
        self.useFixture(fake_config.ConfigFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)
        self.fixed_network_name = 'fixed-net'
        cfg.CONF.set_default('fixed_network_name', self.fixed_network_name,
                             'compute')
        cfg.CONF.set_default('neutron', True, 'service_available')

    @mock.patch.object(test.BaseTestCase, 'get_client_manager')
    @mock.patch.object(test.BaseTestCase, '_get_credentials_provider')
    @mock.patch.object(fixed_network, 'get_tenant_network')
    def test_get_tenant_network(self, mock_gtn, mock_gprov, mock_gcm):
        net_client = mock.Mock()
        mock_prov = mock.Mock()
        mock_gcm.return_value.networks_client = net_client
        mock_gprov.return_value = mock_prov

        test.BaseTestCase.get_tenant_network()

        mock_gcm.assert_called_once_with(credential_type='primary')
        mock_gprov.assert_called_once_with()
        mock_gtn.assert_called_once_with(mock_prov, net_client,
                                         self.fixed_network_name)

    @mock.patch.object(test.BaseTestCase, 'get_client_manager')
    @mock.patch.object(test.BaseTestCase, '_get_credentials_provider')
    @mock.patch.object(fixed_network, 'get_tenant_network')
    @mock.patch.object(test.BaseTestCase, 'get_identity_version')
    @mock.patch.object(credentials, 'is_admin_available')
    @mock.patch.object(clients, 'Manager')
    def test_get_tenant_network_with_nova_net(self, mock_man, mock_iaa,
                                              mock_giv, mock_gtn, mock_gcp,
                                              mock_gcm):
        cfg.CONF.set_default('neutron', False, 'service_available')
        mock_prov = mock.Mock()
        mock_admin_man = mock.Mock()
        mock_iaa.return_value = True
        mock_gcp.return_value = mock_prov
        mock_man.return_value = mock_admin_man

        test.BaseTestCase.get_tenant_network()

        mock_man.assert_called_once_with(
            mock_prov.get_admin_creds.return_value.credentials)
        mock_iaa.assert_called_once_with(
            identity_version=mock_giv.return_value)
        mock_gcp.assert_called_once_with()
        mock_gtn.assert_called_once_with(
            mock_prov, mock_admin_man.compute_networks_client,
            self.fixed_network_name)

    @mock.patch.object(test.BaseTestCase, 'get_client_manager')
    @mock.patch.object(test.BaseTestCase, '_get_credentials_provider')
    @mock.patch.object(fixed_network, 'get_tenant_network')
    def test_get_tenant_network_with_alt_creds(self, mock_gtn, mock_gprov,
                                               mock_gcm):
        net_client = mock.Mock()
        mock_prov = mock.Mock()
        mock_gcm.return_value.networks_client = net_client
        mock_gprov.return_value = mock_prov

        test.BaseTestCase.get_tenant_network(credentials_type='alt')

        mock_gcm.assert_called_once_with(credential_type='alt')
        mock_gprov.assert_called_once_with()
        mock_gtn.assert_called_once_with(mock_prov, net_client,
                                         self.fixed_network_name)

    @mock.patch.object(test.BaseTestCase, 'get_client_manager')
    @mock.patch.object(test.BaseTestCase, '_get_credentials_provider')
    @mock.patch.object(fixed_network, 'get_tenant_network')
    def test_get_tenant_network_with_role_creds(self, mock_gtn, mock_gprov,
                                                mock_gcm):
        net_client = mock.Mock()
        mock_prov = mock.Mock()
        mock_gcm.return_value.networks_client = net_client
        mock_gprov.return_value = mock_prov
        creds = ['foo_type', 'role1']

        test.BaseTestCase.get_tenant_network(credentials_type=creds)

        mock_gcm.assert_called_once_with(roles=['role1'], scope='project')
        mock_gprov.assert_called_once_with()
        mock_gtn.assert_called_once_with(mock_prov, net_client,
                                         self.fixed_network_name)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock


from tempest.lib.services.compute import base_compute_client
from tempest.lib.services.compute import services_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestServicesClient(base.BaseServiceTest):

    FAKE_SERVICES = {
        "services":
        [{
            "status": "enabled",
            "binary": "nova-conductor",
            "zone": "internal",
            "state": "up",
            "updated_at": "2015-08-19T06:50:55.000000",
            "host": "controller",
            "disabled_reason": None,
            "id": 1
        }]
    }

    FAKE_SERVICE = {
        "service":
        {
            "status": "enabled",
            "binary": "nova-conductor",
            "host": "controller"
        }
    }

    FAKE_UPDATE_FORCED_DOWN = {
        "service":
        {
            "forced_down": True,
            "binary": "nova-conductor",
            "host": "controller"
        }
    }

    FAKE_UPDATE_SERVICE = {
        "service": {
            "id": "e81d66a4-ddd3-4aba-8a84-171d1cb4d339",
            "binary": "nova-compute",
            "disabled_reason": "test2",
            "host": "host1",
            "state": "down",
            "status": "disabled",
            "updated_at": "2012-10-29T13:42:05.000000",
            "forced_down": False,
            "zone": "nova"
        }
    }

    def setUp(self):
        super(TestServicesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = services_client.ServicesClient(
            fake_auth, 'compute', 'regionOne')
        self.addCleanup(mock.patch.stopall)

    def test_list_services_with_str_body(self):
        self.check_service_client_function(
            self.client.list_services,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SERVICES)

    def test_list_services_with_bytes_body(self):
        self.check_service_client_function(
            self.client.list_services,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SERVICES, to_utf=True)

    def _test_enable_service(self, bytes_body=False):
        self.check_service_client_function(
            self.client.enable_service,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_SERVICE,
            bytes_body,
            host="nova-conductor", binary="controller")

    def test_enable_service_with_str_body(self):
        self._test_enable_service()

    def test_enable_service_with_bytes_body(self):
        self._test_enable_service(bytes_body=True)

    def _test_disable_service(self, bytes_body=False):
        fake_service = copy.deepcopy(self.FAKE_SERVICE)
        fake_service["service"]["status"] = "disable"

        self.check_service_client_function(
            self.client.disable_service,
            'tempest.lib.common.rest_client.RestClient.put',
            fake_service,
            bytes_body,
            host="nova-conductor", binary="controller")

    def test_disable_service_with_str_body(self):
        self._test_disable_service()

    def test_disable_service_with_bytes_body(self):
        self._test_disable_service(bytes_body=True)

    def _test_log_reason_disabled_service(self, bytes_body=False):
        resp_body = copy.deepcopy(self.FAKE_SERVICE)
        resp_body['service']['disabled_reason'] = 'test reason'

        self.check_service_client_function(
            self.client.disable_log_reason,
            'tempest.lib.common.rest_client.RestClient.put',
            resp_body,
            bytes_body,
            host="nova-conductor",
            binary="controller",
            disabled_reason='test reason')

    def _test_update_service(self, bytes_body=False, status=None,
                             disabled_reason=None, forced_down=None):
        resp_body = copy.deepcopy(self.FAKE_UPDATE_SERVICE)
        kwargs = {}

        if status is not None:
            kwargs['status'] = status
        if disabled_reason is not None:
            kwargs['disabled_reason'] = disabled_reason
        if forced_down is not None:
            kwargs['forced_down'] = forced_down

        resp_body['service'].update(kwargs)

        self.check_service_client_function(
            self.client.update_service,
            'tempest.lib.common.rest_client.RestClient.put',
            resp_body,
            bytes_body,
            service_id=resp_body['service']['id'],
            **kwargs)

    def test_log_reason_disabled_service_with_str_body(self):
        self._test_log_reason_disabled_service()

    def test_log_reason_disabled_service_with_bytes_body(self):
        self._test_log_reason_disabled_service(bytes_body=True)

    def _test_update_forced_down(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_forced_down,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_FORCED_DOWN,
            bytes_body,
            host="nova-conductor",
            binary="controller",
            forced_down=True)

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.11'))
    def test_update_forced_down_with_str_body(self, _):
        self._test_update_forced_down()

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.11'))
    def test_update_forced_down_with_bytes_body(self, _):
        self._test_update_forced_down(bytes_body=True)

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.53'))
    def test_update_service_disable_scheduling_with_str_body(self, _):
        self._test_update_service(status='disabled',
                                  disabled_reason='maintenance')

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.53'))
    def test_update_service_disable_scheduling_with_bytes_body(self, _):
        self._test_update_service(status='disabled',
                                  disabled_reason='maintenance',
                                  bytes_body=True)

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.53'))
    def test_update_service_enable_scheduling_with_str_body(self, _):
        self._test_update_service(status='enabled')

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.53'))
    def test_update_service_enable_scheduling_with_bytes_body(self, _):
        self._test_update_service(status='enabled', bytes_body=True)

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.53'))
    def test_update_service_forced_down_with_str_body(self, _):
        self._test_update_service(forced_down=True)

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.53'))
    def test_update_service_forced_down_with_bytes_body(self, _):
        self._test_update_service(forced_down=True, bytes_body=True)
